/*
 * Spiele programmieren mit Java ME
 * von (c) Gerhard Vlkl (Autor) 
 * Dpunkt Verlag; Auflage: 1 (April 2008) ISBN 389864488X 
 */
import java.io.IOException;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.game.GameCanvas;
import javax.microedition.lcdui.game.LayerManager;
import javax.microedition.lcdui.game.Sprite;
import javax.microedition.lcdui.game.TiledLayer;
import javax.microedition.midlet.MIDlet;

/**
 * Version 1 von FreddyTheFish
 * @author Gerhard Vlkl
 */
public class FreddyTheFishCanvas extends GameCanvas implements Runnable {
    
    /**** Game Model - Start ****/
    private FishActor freddy;
    private Sprite heart;
    private Sprite key;
    private Sprite star;
    private Sprite background;
    private TiledLayer stones;
    private int level = 1;
    private int power = 100;
    private int powerPerTick = 0;
    
    private static final int STATE_RUN=1;
    private static final int STATE_LOST=3;
    private static final int STATE_WIN=4;
    private int gameState;
    /**** Game Model - Ende  ****/
    
    
    MIDlet midlet;
    Thread gameLoop;
    LayerManager layerManager;
    Camera camera;
    private static final int MILLIS_PER_TICK = 50;
    
    
    public FreddyTheFishCanvas(MIDlet aMIDlet) throws IOException{
        super(true); //true: Key-Events unterdrcken
        midlet = aMIDlet;
	init();
    }

    private void init() throws IOException {
        
        //Hintergrund
	background = new Sprite(Image.createImage("/Background.png"));
        
        //Tiled - Hintergrung initialisieren
	stones = new TiledLayer(16,16,Image.createImage("/Stone_Tiles.png"),16,16);
		
        int[][] tiles = {
                        {7, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8},
                        {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
                        {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
                        {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
                        {5, 0, 0, 0, 0, 0, 3, 3, 3, 3, 3, 3, 3, 3, 3, 6},
                        {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
                        {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
                        {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
                        {9, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 0, 0, 0, 0, 4},
                        {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
                        {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
                        {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
                        {5, 0, 0, 0, 0, 0, 0, 0, 0, 6, 7, 7, 7, 7, 7, 7},
                        {5, 0, 0, 0, 0, 0, 0, 6, 7, 7, 7, 7, 7, 7, 7, 7},
                        {5, 0, 0, 0, 0, 0, 6, 7, 7, 7, 7, 7, 7, 7, 7, 7},
                        {0, 0, 0, 0, 0, 6, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7}
            };
		
        for(int i =0;i<16;i++){	
            for(int j= 0;j<16;j++){
        	stones.setCell(j,i,tiles[i][j]);
            }
        }
        
        //Freddy
        freddy = new FishActor(Image.createImage("/Freddy.png"));
        freddy.setPosition(16,background.getHeight()-freddy.getHeight()-4);
        freddy.setSpeed(2);
        freddy.setBorders(0,0,background.getWidth(),background.getHeight());
        
        //Herz initialisieren
	heart = new Sprite(Image.createImage("/Heart.png"));
	heart.setPosition(116,116);
		
	//Gegner Stern initialisieren
	star = new Sprite(Image.createImage("/Star.png"),26,26);
	star.setPosition(getWidth()-50,getHeight()-50);
					
	//Ziel - Schlssel
	key = new Sprite(Image.createImage("/Key.png"));
	key.setPosition(background.getWidth()-key.getWidth()-10,
			10);
        //LayerManager
	layerManager = new LayerManager();
	layerManager.append(freddy);
	layerManager.append(heart);
	layerManager.append(star);
	layerManager.append(key); 
	layerManager.append(stones);
	layerManager.append(background);
        
        camera = new Camera(this,background,getGraphics().getFont().getHeight() + 1);
        
        gameState=STATE_RUN;
    }
    
    protected void showNotify(){
		gameLoop = new Thread(this);
		gameLoop.start();
	}
	
	protected void hideNotify(){
		gameLoop = null;
	}
        
        public void run() {
		Graphics g = getGraphics();
		Thread t = Thread.currentThread();
		
		while(t==gameLoop){
                    long startTime = System.currentTimeMillis();
                    
                    switch(gameState){
                        case STATE_RUN:
                            update(getKeyStates());
                            render(g);
                            break;
                        case STATE_LOST:
                            render(g);
                            g.setColor(255,255,255);
                            g.drawString("VERLOREN!",getWidth()/2,getHeight()/2,Graphics.HCENTER| Graphics.TOP);
                            break;
                        case STATE_WIN:
                            render(g);
                            g.setColor(255,255,255);
                            g.drawString("GEWONNEN!",getWidth()/2,getHeight()/2,Graphics.HCENTER| Graphics.TOP);
                            break;
                    }
                    flushGraphics();
                    long timeTaken = System.currentTimeMillis() - startTime;
			
                    if (timeTaken < MILLIS_PER_TICK) { // etwas warten, damit andere auch dran kommen
                        try{ Thread.sleep(MILLIS_PER_TICK - timeTaken);}
			catch (InterruptedException ex){}
                    }
		}
	}

    private void update(int keyStates) {
        
        freddy.update(keyStates);
        star.nextFrame();
        
        //Zusammensto mit Herz
        if (freddy.collidesWith(heart,true)){
            heart.setVisible(false);  //collidesWith nur, wenn visible
            power+=50;
        }
        
        //Zusammensto mit Star
        if (freddy.collidesWith(star,true)){
            power-=5;
        }
		
        //Zusammensto mit Felsen
        if (freddy.collidesWith(stones,true)){
            freddy.setLastPosition();
            power-=1;
        }
		
        //Zusammensto mit Schlssel
        if(freddy.collidesWith(key,true)){
            gameState=STATE_WIN;   //Gewonnen !!!!!!!!
        }
		
	//Lebensenergie reduzieren
	powerPerTick++;
	if (powerPerTick>50){
            power-=1;
            powerPerTick=0;
        }
	if (power<1) gameState=STATE_LOST;  //Verloren !!!!!!!!
                
       //Layermanager "Kamera" positionieren
        camera.move(freddy.getX(),freddy.getY());
    }

    private void render(Graphics g) {
        //Hintergrund
	g.setColor(0,0,255); //Helles Blau
	g.fillRect(0,0,getWidth(),getHeight());
        
        //Anzeige
	g.setColor(255,255,255);
	g.drawString("Level: " + level,0,0,Graphics.TOP | Graphics.LEFT);
	g.drawString("Power: " + power,getWidth(),0,Graphics.TOP | Graphics.RIGHT);
        
        //Sprites
        layerManager.paint(g,0,camera.abstandVonOben);
        layerManager.setViewWindow(camera.x,camera.y,camera.width,camera.height);
    }
}
