/*
 * Spiele programmieren mit Java ME
 * von (c) Gerhard Vlkl (Autor) 
 * Dpunkt Verlag; Auflage: 1 (April 2008) ISBN 389864488X 
 */
package mgame.ui;

import java.io.IOException;
import java.io.InputStream;
import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Graphics;
import javax.microedition.m2g.SVGImage;
import javax.microedition.m2g.ScalableGraphics;
import org.w3c.dom.Document;
import org.w3c.dom.svg.SVGElement;
import org.w3c.dom.svg.SVGRGBColor;
import org.w3c.dom.svg.SVGSVGElement;

/**
 * FreddyTheFish mit GUI
 * @author Gerhard Vlkl
 */
public class SVGMainMenuDialog extends Canvas implements CommandListener,IMainMenuDialog{
     private UiManager uiManager;
     private int currentItem=1;

    private ScalableGraphics sg;

    private SVGImage image;
    private SVGSVGElement root=null;
    private SVGRGBColor currentColor;
    private SVGRGBColor normalColor;
    
    private static final int MAXITEM=7;
    
       public SVGMainMenuDialog(UiManager uiManager) {
        this.uiManager = uiManager;
        
        addCommand(new Command(uiManager.SELECT,Command.ITEM,1));
        setCommandListener(this);
        
        sg = ScalableGraphics.createInstance();
        try {
            InputStream svgStream = getClass().getResourceAsStream("/HauptMenu.SVG");
            image = (SVGImage) SVGImage.createImage(svgStream,null);
            
            Document document = image.getDocument();
            root = (SVGSVGElement)document.getDocumentElement();
            currentColor=root.createSVGRGBColor(100,0,0);
            normalColor=root.createSVGRGBColor(220,220,220);
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        setCurrentItem(1);
    }

    protected void keyPressed(int keyCode) {
        int gameAction = getGameAction( keyCode );
        switch ( gameAction ) {
            case Canvas.UP:
                setCurrentItem(getPrevItem());
                repaint();
                break;
            case Canvas.DOWN:
                setCurrentItem(getNextItem());
                repaint();
                break;
        }
    }
    
    private int getNextItem(){
        return currentItem==MAXITEM ? 1 : currentItem+1;
    }
    
    private int getPrevItem(){
        return currentItem==1 ? MAXITEM : currentItem-1;
    }
    
    private void setCurrentItem(int newValue){
        SVGElement textAlt = getText(currentItem);
        textAlt.setRGBColorTrait("fill", normalColor);
        SVGElement textNeu = getText(newValue);
        textNeu.setRGBColorTrait("fill",currentColor);
        currentItem=newValue;
    }
    
    private SVGElement getText(int item){
        Document document = image.getDocument();
        return (SVGElement)document.getElementById("text_" + item);
    }
    
    protected void paint(Graphics g) {
	sg.bindTarget(g);
        sg.render(0,0,image);
        sg.releaseTarget();
    }

    public void commandAction(Command command, Displayable displayable) {
        if(displayable!=this) return;
        
        switch(currentItem){
            case 1:
                uiManager.showNewGameDialog();
                break;
            case 2:
                uiManager.showOptionDialog();
                break;
            case 3:
                uiManager.showHighScoreDialog();
                break;
            case 4:
                uiManager.showHelpDialog();
                break;
            case 5:
                uiManager.showAboutDialog();
                break;
            case 7:
                uiManager.getGame().restartGame();
                break;
            case 6:
                uiManager.getGame().exitGame();
                break;
        }

    }
    
    public void showNotify(){
    /*    if(game!=null && this.size()!=0){
            if(game.isRunning()){
                if (this.getString(0)!=RESTART){
                    this.insert(0,RESTART,null);
                }
            }
            else{
                if(this.getString(0)==RESTART){
                    this.delete(0);
                }
            }
        }*/
    }
    
}
