/*
 * Spiele programmieren mit Java ME
 * von (c) Gerhard Vlkl (Autor) 
 * Dpunkt Verlag; Auflage: 1 (April 2008) ISBN 389864488X 
 */
package mgame;

import java.io.IOException;
import javax.microedition.lcdui.Image;
import javax.microedition.midlet.MIDlet;
import mgame.ui.GameDialog;
import mgame.ui.UiManager;
/**
 * FreddyTheFish mit Audio
 * @author Gerhard Vlkl
 */
public abstract class Game {
    
    public OptionManager optionManager;
    public UiManager uiManager;
    public HighScoreManager highScoreManager;
    public AudioManager audioManager;
    public MIDlet midlet;
    private GameDialog gameDialog=null;
    private boolean firstTime=true;
    
    
    /** Creates a new instance of Game */
    public Game(MIDlet midlet) {
        this.midlet = midlet;
        optionManager = new OptionManager();
        uiManager = new UiManager(this);
        highScoreManager = new HighScoreManager(this);
        audioManager = new AudioManager(this);
    }
    
    public void restartGame(){
        uiManager.showGameDialog();
    }
    
    public boolean isRunning(){
        if (gameDialog==null) return false;
        
        return (gameDialog.getGameState()==GameDialog.STATE_RUN);
    }
    
    public void exitGame() {
        //midlet.destroyApp(false);
        destroyApp();
        midlet.notifyDestroyed();
    }
    
    public void startNewGame(boolean multiuser) {
        try{
            gameDialog = getNewGameDialog();
            uiManager.setGameDialog(gameDialog);
            uiManager.showGameDialog();
        } catch (Exception ex) {
            ex.printStackTrace();
        };
    }
    
    //**** MIDlet Facade
    public void startApp(){
        if (firstTime){
            try{
                audioManager.playTitleMusic();
                uiManager.showSplashDialog(Image.createImage(Image.createImage(getSplashImage())));
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }
    
    public void pauseApp() {
        audioManager.pauseApp();
    }
    
    public void destroyApp(){
        audioManager.destroyApp();
        optionManager.destroyApp();
    }
    
    //**** Optionen
    public void setMusicOn(boolean on){
        if (on){
            optionManager.set(optionManager.OPTION_AUDIO_ON,"ON");
        }else{
            optionManager.set(optionManager.OPTION_AUDIO_ON,"OFF");
            audioManager.destroyApp();
        }
    }
    
    public void setVolume(int volume){
        optionManager.set(optionManager.OPTION_AUDIO_VOLUME,String.valueOf(volume));
        audioManager.setVolume(volume);
    }
    //**** Individuelle Attribute
    protected abstract GameDialog getNewGameDialog() throws IOException;
    protected abstract String getTitleMusic();
    protected abstract String getSplashImage();
}
