/*
 * Spiele programmieren mit Java ME
 * von (c) Gerhard Vlkl (Autor) 
 * Dpunkt Verlag; Auflage: 1 (April 2008) ISBN 389864488X 
 */

package mgame.ui;

import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.game.GameCanvas;
import mgame.Game;

/**
 * FreddyTheFish mit Audio
 * @author Gerhard Vlkl
 */
public class UiManager {
    
    private Game game;
    
    private GameCanvas gameDialog;
    private IMainMenuDialog mainMenueDialog=null;
    private NewGameDialog newGameDialog=null;
    private HelpTextDialog helpTextDialog=null;
    private HelpMenuDialog helpMenuDialog=null;
    private InputHighScoreDialog inputHighScoreDialog=null;
    private SplashDialog splashDialog=null;
    private OptionDialog optionDialog=null;
    private AboutDialog aboutDialog=null;
    private SystemInfoDialog systemInfoDialog=null;
    private OptionMusicDialog optionMusicDialog=null;
    private OptionVolumeDialog optionVolumeDialog=null;
    
    String helpChapters[];
    String helpTexts[];

    public static final String SELECT="Auswhl.";
    public static final String NEW_GAME="Neues Spiel";
    public static final String OPTIONS="Einstellungen";
    public static final String HIGHSCORE="Highscores";
    public static final String HELP="Anleitung";
    public static final String ABOUT="Info";
    public static final String EXIT="Beenden";
    public static final String RESTART="Fortfahren";
    public static final String BACK="Zurck";
    public static final String OK="Ok";
    public static final String SYSINFO="System-Info";


    public UiManager(Game game) {
        this.game = game;
    }

    private void setCurrentDialog(Displayable displayable){
        Display.getDisplay(game.midlet).setCurrent(displayable);
    }
    
    public Game getGame(){
        return this.game;
    }
    
    /** Hilfe-Dialog **/
    public void setHelpChapters(String chapters[]){
        this.helpChapters = chapters;
    }
    
    public void setHelpTexts(String texts[]){
        this.helpTexts = texts;
    }
    
    void showHelpText(int selectedItem) {
       if (this.helpTextDialog==null){
           this.helpTextDialog= new HelpTextDialog(this);
       }
       this.helpTextDialog.setText(this.helpTexts[selectedItem]);
       setCurrentDialog(this.helpTextDialog);
    }
    
    /** Game-Dialog **/
    public void setGameDialog(GameCanvas aGameDialog) {
        gameDialog = aGameDialog;
    }

    public void showGameDialog() {
        setCurrentDialog(gameDialog);
    }

    /** Splash-Dialog **/
    public void setSplashDialog(SplashDialog splashDialog){
        this.splashDialog = splashDialog;
    }
    public void showSplashDialog(Image image) {
        if(this.splashDialog == null){
            this.splashDialog = new SplashDialog(this,image);
        }
         Display.getDisplay(game.midlet).setCurrent(splashDialog);
    }
    
    /** Hauptmen-Dialog**/
    public void setMainMenuDialog(IMainMenuDialog menueDialog){
        this.mainMenueDialog = menueDialog;
    }
    
    public void showMainDialog() {
        if (this.mainMenueDialog == null){
            this.mainMenueDialog = new MainMenuDialog(this);
        }
        setCurrentDialog((Displayable)this.mainMenueDialog);
        this.mainMenueDialog.showNotify();
    }
    
    /** Option-Dialog **/
    void showOptionDialog(){
         if (this.optionDialog == null){
            this.optionDialog = new OptionDialog(this);
        }
        setCurrentDialog((Displayable)this.optionDialog);
    }
    
    /** HighScore-Dialog **/
    void showHighScoreDialog(){
        setCurrentDialog(new HighScoreDialog(this));
    }
    
    public void showInputHighScoreDialog(long score) {
       if (this.inputHighScoreDialog==null){
           this.inputHighScoreDialog = new InputHighScoreDialog(this);
       }
       this.inputHighScoreDialog.setScore(score);
       setCurrentDialog(this.inputHighScoreDialog);
    }
      
    /** Hilfe-Dialog **/
    void showHelpDialog(){
        if (this.helpMenuDialog==null){
            this.helpMenuDialog = new HelpMenuDialog(this,this.helpChapters);
        }
        setCurrentDialog(this.helpMenuDialog);    
    }
    
    /** Info-Dialog **/
    void showAboutDialog(){
          if (this.aboutDialog==null){
            this.aboutDialog = new AboutDialog(this);
        }
        setCurrentDialog(this.aboutDialog);   
    }
    
    public void setAboutDialog(AboutDialog aboutDialog) {
        this.aboutDialog = aboutDialog;
    }
 
    /** Multi-UserDialog **/
    void showMultiUserDialog() {
     
    }

    /** New Game-Dialog **/
    void showNewGameDialog() {
        if (this.newGameDialog==null){
            this.newGameDialog = new NewGameDialog(this);
        }
        setCurrentDialog(this.newGameDialog);
    }    

    void showSystemInfoDialog() {
        if (this.systemInfoDialog==null){
            this.systemInfoDialog = new SystemInfoDialog(this);
        }
        setCurrentDialog(this.systemInfoDialog);
    }

    void showOptionMusic() {
         if (this.optionMusicDialog==null){
            this.optionMusicDialog = new OptionMusicDialog(this);
        }
        setCurrentDialog(this.optionMusicDialog);
    }

    void showOptionVolume() {
         if (this.optionVolumeDialog==null){
            this.optionVolumeDialog = new OptionVolumeDialog(this);
        }
        setCurrentDialog(this.optionVolumeDialog);
    }
}
