/*
 * Spiele programmieren mit Java ME
 * von (c) Gerhard Vlkl (Autor) 
 * Dpunkt Verlag; Auflage: 1 (April 2008) ISBN 389864488X 
 */
import java.io.IOException;
import java.io.InputStream;
import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.List;
import javax.microedition.media.Manager;
import javax.microedition.media.MediaException;
import javax.microedition.media.Player;
import javax.microedition.media.control.MIDIControl;
import javax.microedition.media.control.PitchControl;
import javax.microedition.media.control.VolumeControl;
import javax.microedition.midlet.MIDlet;

/**
 * Soundsamples
 * @author Gerhard Vlkl
 */
public class MenuDialog extends List implements CommandListener{
    
    public MIDlet midlet;
    
    public MenuDialog(MIDlet midlet) {
        super("AudioTest",List.IMPLICIT,new String[]{"INFO",
        "PLAY_WAV",
        "PLAY_MIDI",
        "PLAY_HALF",
        "PLAY_HIGH",
        "MIDI_EVENT",
        "3D-AUDIO",
        "ENDE"},null);
        this.midlet = midlet;
        
        setSelectCommand(new Command("AUSWAHL",Command.ITEM,1));
        setCommandListener(this);
    }
    
    public void commandAction(Command command, Displayable displayable) {
        if(displayable!=this) return;
        
        String selectedItem = this.getString(this.getSelectedIndex());
        
        if (selectedItem.equals("ENDE")){
            midlet.notifyDestroyed();
        }else if (selectedItem.equals("INFO")){
            Display.getDisplay(midlet).setCurrent(new InfoDialog(this)) ;
        }else if (selectedItem.equals("PLAY_WAV")){
            try {
                playWav();
           } catch (Exception ex) {
                showException(ex);
           }
        }else if (selectedItem.equals("PLAY_MIDI")){
            try {
                playMIDI();
            } catch (Exception ex) {
                showException(ex);
            }
        }else if (selectedItem.equals("PLAY_HALF")){
            try {
                playWavHalf();
           } catch (Exception ex) {
                showException(ex);
            }
        }else if (selectedItem.equals("PLAY_HIGH")){
            try {
                playHighMIDI();
           } catch (Exception ex) {
                showException(ex);
            }
        }else if (selectedItem.equals("MIDI_EVENT")){
            try {
                doMIDIEvent();
            } catch (Exception ex) {
                showException(ex);
            }
        }else if (selectedItem.equals("3D-AUDIO")){
            Display.getDisplay(midlet).setCurrent(new ThreeDDialog(this)) ;
        }
    }
    
    private void showException(Exception ex){
        Alert info = new Alert("Exception",ex.toString(),null,null);
        info.setTimeout(Alert.FOREVER);
        Display.getDisplay(midlet).setCurrent(info);
    }
    
    private void doMIDIEvent() throws IOException, MediaException, InterruptedException{
        Player p = Manager.createPlayer(Manager.MIDI_DEVICE_LOCATOR);
        p.prefetch();
        MIDIControl mc = (MIDIControl) p.getControl("MIDIControl");
        mc.setProgram(0,0,0);
        mc.shortMidiEvent(MIDIControl.NOTE_ON | 0x0, 60,100);
        Thread.sleep(1000);
        mc.shortMidiEvent(MIDIControl.NOTE_ON| 0x0, 60,0);
    }
    
    private void playWav() throws IOException, MediaException{
        InputStream is = getClass().getResourceAsStream("/test.wav");
        Player p = Manager.createPlayer(is,"audio/x-wav");
        p.start();
    }
    
    private void playWavHalf() throws IOException, MediaException{
        InputStream is = getClass().getResourceAsStream("/test.wav");
        Player p = Manager.createPlayer(is,"audio/x-wav");
        p.realize();
        VolumeControl vc = (VolumeControl) p.getControl("VolumeControl");
        vc.setLevel(50);
        p.start();
    }
    
    private void playMIDI() throws IOException, MediaException{
        InputStream is = getClass().getResourceAsStream("/test.mid");
        Player p = Manager.createPlayer(is,"audio/midi");
        p.setLoopCount(2);
        p.start();
    }
    
    private void playHighMIDI() throws IOException, MediaException{
        InputStream is = getClass().getResourceAsStream("/test.mid");
        Player p = Manager.createPlayer(is,"audio/midi");
        p.realize();
        PitchControl pc = (PitchControl)p.getControl("PitchControl");
        if (pc!=null){
            pc.setPitch(2000);
        }
        p.start();
    }
}
