/*
 * Spiele programmieren mit Java ME
 * von (c) Gerhard Vlkl (Autor) 
 * Dpunkt Verlag; Auflage: 1 (April 2008) ISBN 389864488X 
 */

import java.io.IOException;
import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.game.LayerManager;
import javax.microedition.lcdui.game.Sprite;
import javax.microedition.lcdui.game.TiledLayer;
import mgame.Game;
import mgame.ui.GameDialog;

/**
 * NelsonFlow
 * @author Gerhard Vlkl
 */
public class NelsonFlowCanvas extends GameDialog implements Runnable { 
    
    /**** Game Model - Start ****/
    private Player player[] = new Player[2];
    private Player myPlayer = null;
    private Player myOpponent = null;
    private Sprite dot;
    private Sprite background;
    private TiledLayer stones;
    private int level = 1;
    private int power = 100;
    private int powerPerTick = 0;
    private static final String PLAYLIST[] ={"/Level1Music.mid",
    "/Bounce.mid",
    "/Winner.mid",
    "/Looser.mid"};
    
    private static final int SND_BACKGROUND=0;
    private static final int SND_BOUNCE=1;
    private static final int SND_WINNER=2;
    private static final int SND_LOOSER=3;
    
    /**** Game Model - Ende  ****/
    volatile Thread gameLoop;
    LayerManager layerManager;
    Camera camera;
    private static final int MILLIS_PER_TICK = 100;
    
    private boolean highScoreChecked=false;
    private boolean firstCall=true;

    private int boardWidth;
    private int boardHeight;
    
    
    public NelsonFlowCanvas(Game game,boolean multiuser) throws IOException{
        super(game,multiuser);
        init();
    }
    
    private void init() throws IOException {       
        //Tiled - Hintergrung initialisieren
        stones = new TiledLayer(16,32,Image.createImage("/Water_Tiles.png"),16,16);
        boardWidth=16*16;
        boardHeight=16*32;
                
        int[][] tiles = {
            {7, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 3, 3, 3, 3, 3, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 3, 3, 3, 3, 3, 3, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 3, 3, 3, 3, 3, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 3, 3, 3, 3, 3, 3, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4},
            {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
        };
        
        for(int i =0;i<32;i++){
            for(int j= 0;j<16;j++){
                stones.setCell(j,i,tiles[i][j]);
            }
        }
        
        //Player
        player[0] = new Player(Image.createImage("/Player_0.png"));
        player[0].setPosition(16,boardHeight-player[0].getHeight()-8);
        player[0].setSpeed(2);
        player[0].setBorders(0,0,boardWidth,boardHeight);
        
        if (multiuser){
            player[1] = new Player(Image.createImage("/Player_1.png"));
            player[1].setPosition(32,boardHeight-player[1].getHeight()-8);
            player[1].setSpeed(2);
            player[1].setBorders(0,0,boardWidth,boardHeight);
        }else{
            player[1]=null;
            myPlayer=player[0];
        }
        
        //Ziel - Dot
        dot = new Sprite(Image.createImage("/Dot.png"));
        dot.setPosition(boardWidth-dot.getWidth()-10,10);
        
        //LayerManager
        layerManager = new LayerManager();
        layerManager.append(player[0]);
        if(player[1]!=null)layerManager.append(player[1]);
        layerManager.append(dot);
        layerManager.append(stones);
        
        camera = new Camera(this,boardWidth,boardHeight,getGraphics().getFont().getHeight() + 1);
        
        //Audio
        game.audioManager.setPlayList(PLAYLIST);
    }
    
  
    
    protected void showNotify(){
        gameLoop = new Thread(this);
        gameLoop.start();
    }
    
    protected void hideNotify(){
        gameLoop = null;
    }
    
    public synchronized void setLeader(boolean isLeader){
        if (isLeader){
            myPlayer   = player[0];
            myOpponent = player[1];
        } else {
            myOpponent = player[0];
            myPlayer   = player[1];
        }
    }
    
    public void run() {
        Graphics g = getGraphics();
        try {
            
            game.audioManager.playBackground(SND_BACKGROUND);
            
            if (firstCall) {
                initFirstCall();
                firstCall=false;
            }
            
            Thread t = Thread.currentThread();
            
            while(t==gameLoop){
                long startTime = System.currentTimeMillis();
                
                switch(gameState){
                    case STATE_START:
                        render(g);
                        g.setColor(0,0,0);
                        g.drawString("Start...",getWidth()/2,getHeight()/2,Graphics.HCENTER| Graphics.TOP);
                        break;
                    case STATE_RUN:
                        update(getKeyStates());
                        render(g);
                        break;
                    case STATE_LOST:
                        render(g);
                        g.setColor(0,0,0);
                        g.drawString("VERLOREN!",getWidth()/2,getHeight()/2,Graphics.HCENTER| Graphics.TOP);
                        game.audioManager.stop();
                        break;
                    case STATE_WIN:
                        render(g);
                        g.setColor(0,0,0);
                        g.drawString("GEWONNEN!",getWidth()/2,getHeight()/2,Graphics.HCENTER| Graphics.TOP);
                        game.audioManager.stop();
                        break;
                    case STATE_INTERRUPTED:
                        render(g);
                        g.setColor(0,0,0);
                        g.drawString("SPIEL ABGEBROCHEN!",getWidth()/2,getHeight()/2,Graphics.HCENTER| Graphics.TOP); 
                        break;
                }
                flushGraphics();
                long timeTaken = System.currentTimeMillis() - startTime;
                
                if (timeTaken < MILLIS_PER_TICK) { // etwas warten, damit andere auch dran kommen
                    try{ Thread.sleep(MILLIS_PER_TICK - timeTaken);} catch (InterruptedException ex){}
                }
            }
            
            game.audioManager.stop();
        } catch(Exception e){
            e.printStackTrace();
            Alert info = new Alert("Exception",e.toString(),null,null);
            info.setTimeout(Alert.FOREVER);
            Display.getDisplay(game.midlet).setCurrent(info);
        }
    }
    
    private void testHighScore(){
        if(!highScoreChecked){
            game.highScoreManager.checkHighScore(power);
            highScoreChecked=true;
        }
    }
    
    private void update(int keyStates) {
        
        myPlayer.update(keyStates);
               
        //Zusammensto mit Felsen
        if (myPlayer.collidesWith(stones,true)){
            myPlayer.setLastPosition();
            power-=1;
        }
        
        
        //Zusammensto mit Schlssel
        if(myPlayer.collidesWith(dot,true)){
            gameState=STATE_WIN;   //Gewonnen !!!!!!!!
        }
        
        //Lebensenergie reduzieren
        powerPerTick++;
        if (powerPerTick>50){
            power-=1;
            powerPerTick=0;
        }
        if (power<1) gameState=STATE_LOST;  //Verloren !!!!!!!!
        
        if (multiuser) game.communityManager.netSynchronizer.update(myPlayer.getX(), myPlayer.getY(),gameState);
        
        //Layermanager "Kamera" positionieren
        camera.move(myPlayer.getX(),myPlayer.getY());
    }
    
    synchronized public void setOpponentPosition(int x, int y){
        System.out.println("Opponent position "+myOpponent.getX()+"-->"+x+" "+myOpponent.getY()+"-->"+y);
        myOpponent.setPosition(x,y);
    }
    
    private void render(Graphics g) {
        //Hintergrund
        g.setColor(255,255,255); //Weiss
        g.fillRect(0,0,getWidth(),getHeight());
        
        //Anzeige
        g.setColor(0,0,255);
        g.drawString("Level: " + level,0,0,Graphics.TOP | Graphics.LEFT);
        g.drawString("Power: " + power,getWidth(),0,Graphics.TOP | Graphics.RIGHT);
        
        //Sprites
        layerManager.paint(g,0,camera.abstandVonOben);
        layerManager.setViewWindow(camera.x,camera.y,camera.width,camera.height);
    }
}
