/*
 * Spiele programmieren mit Java ME
 * von (c) Gerhard Vlkl (Autor) 
 * Dpunkt Verlag; Auflage: 1 (April 2008) ISBN 389864488X 
 */

package mgame;

import java.io.IOException;
import java.io.InputStream;
import javax.microedition.media.Manager;
import javax.microedition.media.MediaException;
import javax.microedition.media.Player;
import javax.microedition.media.control.VolumeControl;

/**
 * NelsonFlow
 * @author Gerhard Vlkl
 */
public class AudioManager {
    
    private Game game;
    private static final int MAX_PLAYERS=10;
    private Player[] players = new Player[MAX_PLAYERS];
    private Player backgroundPlayer=null;
    private String[] playList=null;
    private int volume = -1;
    
    public AudioManager(Game game) {
        this.game = game;
    }
    
    public void setPlayList(String[] playList) {
        this.playList = playList;
        for(int i=1;i<MAX_PLAYERS;i++){
            if(players[i]!=null){
                players[i].close();
                players[i]=null;
            }
        }
        if(backgroundPlayer!=players[0]){
            backgroundPlayer=null;
        }
    }
    
    public void playTitleMusic(){
        playSound(0,true);
    }
    
    public synchronized void stop() {
        for(int i=0;i<MAX_PLAYERS;i++){
            if(players[i]!=null){
                try {
                    players[i].stop();
                } catch (MediaException ex) {
                    ex.printStackTrace();
                }
            }
        }
    }
    
    
    
    public void playBackground(int sound) {
        playSound(sound+1,true);
    }
    
    public void play(int sound) {
        playSound(sound+1,false);
    }
    
    private void playSound(int sound, boolean background) {
        //Sound ja/nein
         if(game.optionManager.get(game.optionManager.OPTION_AUDIO_ON).equals("ON")){
            if (players[sound]==null){
                String title;
                if(sound==0){
                    title=game.getTitleMusic();
                }else{
                    title=playList[sound-1];
                }
                players[sound]=createPlayer(title);
            }
            if(players[sound].getState()!=Player.STARTED){
                if(background){
                    if(backgroundPlayer!=null && backgroundPlayer!=players[sound]){
                        try {
                            backgroundPlayer.stop();
                        } catch (MediaException ex) {
                            ex.printStackTrace();
                        }
                    }
                    backgroundPlayer = players[sound];
                    players[sound].setLoopCount(-1);
                }
                try {
                    if(volume==-1){
                        volume=Integer.parseInt(game.optionManager.get(OptionManager.OPTION_AUDIO_VOLUME));
                    }
                    players[sound].realize();
                    VolumeControl v = (VolumeControl) players[sound].getControl("VolumeControl");
                    v.setLevel(volume);
                    players[sound].start();
                } catch (MediaException ex) {
                    ex.printStackTrace();
                }
            }
        }
    }
    
    private Player createPlayer(String sound) {
        String ext = sound.substring(sound.length()-3);
        String type="";
        
        if(ext.equals("mid")){
            type = "audio/midi";
        }else if(ext.equals("wav")){
            type = "audio/x-wav";
        }
        InputStream is = getClass().getResourceAsStream(sound);
        Player p = null;
        try {
            p = Manager.createPlayer(is,type);
        } catch (IOException ex) {
            ex.printStackTrace();
        } catch (MediaException ex) {
            ex.printStackTrace();
        }
        return p;
    }
    
    public void destroyApp() {
        for(int i=0;i<MAX_PLAYERS;i++){
            if(players[i]!=null){
                players[i].close();
            }
        }
    }
    
    public void pauseApp() {
        stop();
    }

    void setVolume(int volume) {
        this.volume = volume;
         for(int i=0;i<MAX_PLAYERS;i++){
            if(players[i]!=null){
                 VolumeControl v = (VolumeControl) players[i].getControl("VolumeControl");
                 v.setLevel(volume);
            }
        }
    }
}
