/*
 * Spiele programmieren mit Java ME
 * von (c) Gerhard Vlkl (Autor) 
 * Dpunkt Verlag; Auflage: 1 (April 2008) ISBN 389864488X 
 */
package mgame;

/**
 * NelsonFlow
 * @author Gerhard Vlkl
 */
import com.nokia.sm.net.ItemList;
import com.nokia.sm.net.ServerComm;
import com.nokia.sm.net.SnapEventListener;

import java.util.Vector;
import mgame.ui.GameDialog;
import mgame.ui.LoginDialog;

public class CommunityManager implements SnapEventListener,Runnable,CommunityCommandListener {
     
    public static final String NON_CRITICAL = "non-critical";
    
    private Game game;
    private GameDialog gameDialog;
    
    private Vector cmdList;
    private ServerComm serverComm=null;
    volatile Thread cmdLoop=null;
    
    private String lastError;        //Fr Fehlerbehandlung
    private int lastErrorSeverity;
    
    private CommunityCommandListener asyncListener = null; //Fehler-Listener fr asynchorne Commands
    private String asyncCommand = null;
    
    //Snap specific data
    private final Integer gcid = new Integer(49152);
    private String webSessionID=null;
    private String impsSessionID=null;
    private String snapSessionID=null;
    private String snapUserID=null;

    public NetSynchronizer netSynchronizer=null;
                
    public CommunityManager(Game game) {
        this.game = game;
        cmdList = new Vector();
    }

    /* Neues Spiel starten */
    void startNewGame(GameDialog gameDialog) {
        this.gameDialog = gameDialog;
                
        //Command Verarbeitung starten
        cmdLoop = new Thread(this);
        cmdLoop.start();
        //Ggf. Login starten
        if (!isLoggedIn()){
            game.uiManager.showLoginDialog();
        }
        //Spiel starten
    }
    
    private boolean isLoggedIn() {
        return webSessionID != null;
    }
    
    public void handleLogin(String username, String password, boolean remember, LoginDialog dialog){
        ItemList itemList = new ItemList();
        itemList.setItem("cmd", "extendedLogin");
        itemList.setItem("user", username);
        itemList.setItem("pass", password);
        itemList.setItem("gcid", this.gcid);
        itemList.setItem("presence", "");
        itemList.setItem("listener", this);
        
        /*if (remember) {
            saveRmsValue( RMS_USERNAME, username);
            saveRmsValue( RMS_PASSWORD, password);
        } else {
            saveRmsValue( RMS_USERNAME, "");
            saveRmsValue( RMS_PASSWORD, "");
        }*/
        
      //  this.username = username;
        this.addCmd(itemList);
    }
    
    public void resultLogin(String errorMessage, ItemList results){
        if (errorMessage != null) {

            if (errorMessage.equals("WebServiceAuthFailure 965") ||
                    errorMessage.equals("WebServiceInvalidData 973")) {
                this.showError(
                        "Login nicht erfolgreich. UserID oder Passwort waren " +
                        "nicht richtig. " +
                        "Bitte prfen Sie, ob die UserID bereits erzeugt wurde!");
                return;    
            } else if (errorMessage.equals("WebServiceUnknownUser 966")) {
                this.showError(
                        "Die UserID gibt es nicht." +
                        "Bitte prfne Sie, ob die UserID bereits erzeugt wurde!");
                return;
            }
        }else {
            setLoginInfo(results.getString("webSessionID"),
                         results.getString("impsSessionID"),
                         results.getString("snapSessionID"),
                         results.getString("snapUserID"));
            handleQuickMatch();
                //buddyList.set(results.getList("buddyList"));
                //switchToView(Community.HOME);
        }
    }
    
    public void setLoginInfo(String webSessionID, String impsSessionID, String snapSessionID, String snapUserID) {
        this.webSessionID = webSessionID;
        this.impsSessionID = impsSessionID;
        this.snapSessionID = snapSessionID;
        this.snapUserID = snapUserID;
    }
    
    public void handleQuickMatch() {
        ItemList il = new ItemList();
        il.setItem("cmd", "randomStart");
        il.setItem("lobbyID", "Random");
        il.setItem("lobbyMaxUsers", new Integer(0)); // Note: 0 = infinite
        il.setItem("gameRoomMaxUser", new Integer(2));
        il.setItem("listener", this);

        addCmd(il);
    }
    
    public void resultQuickMatch(String errorMessage, ItemList results){
       // gameDialog.setOpponent(getOpponentName(results));
        this.netSynchronizer = new NetSynchronizer(this,game.getGameDialog());
        System.out.println("CommunityManager: NetSynchronizer created");
        game.uiManager.showGameDialog();
        return;
    }
    
    public synchronized  void addCmd(ItemList cmd) {
        if (cmdLoop!=null) {
            cmdList.addElement(cmd);
            notifyAll();
        } else {
            System.out.println("CommunityManager.addCmd():");
            System.out.println("Event loop finished (most likely fatally), cannot execute command!");
        }
    }
    
    public synchronized void cancelNonCriticalCmds() {
        ItemList il;

        for (int i = 0; i < cmdList.size(); i++) {
            il = (ItemList) cmdList.elementAt(i);

            if (il.getString(NON_CRITICAL) != null) {
                cmdList.removeElementAt(i--);
            }
        }
    }
 
    // Hauptschleife
    public void run() {
        ItemList itemList, returnValues;
        CommunityCommandListener listener;
        String error, cmd;
        
        try {
           Thread t = Thread.currentThread();
            
            while(t==cmdLoop){
                synchronized (this) {
                    while (cmdList.isEmpty()) { //Warten, bis command in Liste
                        try {wait();} catch (InterruptedException e) {}
                    }
                    
                    synchronized (cmdList) { // Nchsten command lesen
                        itemList = (ItemList)cmdList.elementAt(0);
                        cmdList.removeElementAt(0);
                    }
                }
                cmd = itemList.getString("cmd");
                returnValues = null;
                lastError = null;
                lastErrorSeverity = -1;
                
                System.out.println("CommunityManager: cmd = " + cmd);

                //Synchrone Commands
                if (cmd.equals("extendedLogin") || cmd.equals("createUser")) {
                    try {
                        if (serverComm == null) {
                            System.out.println("ServerComm erzeugen");
                            serverComm = new ServerComm(); //Communication mit SNAP
                            serverComm.addSnapEventListener(this); //Dieses Object als Listener registrieren
                        }

                        if (cmd.equals("extendedLogin")) {
                            returnValues = serverComm.extendedLogin(
                                  itemList.getString("user"),
                                  itemList.getString("pass"),
                                  itemList.getInteger("gcid"),
                                  itemList.getString("presence"),
                                  false
                                  );
                        }/* else if (cmd.equals("createUser")){
                            returnValues = serverComm.createUser(
                                  itemList.getString("user"),
                                  itemList.getString("pass"),
                                  null, null,
                                  itemList.getString("email"),
                                  itemList.getString("dob"),
                                  null, null,
                                  operatorId
                                  );
                        }*/
                    } catch (RuntimeException r) {
                        if (r.toString().toLowerCase().indexOf("security")!=-1) {
                            showError(
                                  "Verbingung zum Netzwerk kann nicht hergestellt" +
                                  "werden. Sie mssen 'JA' drcken, " +
                                  "um die Verbinung zu GPRS zu erlauben. " +
                                  "Bitte starten Sie die Applikation erneut!");
                            continue;
                        }
                    } catch (Exception e) {
                        showError( "Bei der Verbindung zu SNAP gab es Probleme."
                              + "Bitte prfen Sie device network profile "
                              + "oder permission settings."
                              );
                        continue;
                    }
                    
                   //QuickMatch-Start
                } else if (cmd.equals("randomStart")) {
                  //  waitingForGame = true;
                    returnValues = serverComm.randomStart(
                          itemList.getString("lobbyID"),
                          itemList.getInteger("lobbyMaxUsers"),
                          itemList.getInteger("gameRoomMaxUser")
                          );
                        
                    // Get top rankings
                }/* else if (cmd.equals("topRankings")) {
                    returnValues = comm.getTopRankings(
                          itemList.getInteger("gcid"),
                          itemList.getString("stat"),
                          itemList.getInteger("count"),
                          null,
                          null
                          );
                    
                    // Get player statistics
                } else if (cmd.equals("playerStatistics")) {
                    returnValues = comm.getPlayerStatistics(
                          itemList.getInteger("gcid"),
                          null,
                          null,
                          null,
                          null
                          );
                    
                    // Send a friend message
                } else if (cmd.equals("sendBuddyMessage")) {
                    comm.sendBuddyMessage(
                          itemList.getString("name"),
                          itemList.getString("msg")
                          );
                    
                    // Accept another user's friend request
                } else if (cmd.equals("acceptBuddyRequest")) {
                    comm.acceptBuddyRequest(itemList.getString("name"));
                    
                    // Decline another user's friend request
                } else if (cmd.equals("declineBuddyRequest")) {
                    comm.declineBuddyRequest(itemList.getString("name"));
                    
                    // Ask another user to be your friend
                } else if (cmd.equals("requestBuddy")) {
                    comm.requestBuddy(
                          itemList.getString("name"),
                          itemList.getString("message")
                          );
                    
                    // Remove a friend
                } else if (cmd.equals("removeBuddy")) {
                    comm.removeBuddy(itemList.getString("name"));
                    
                    // Stop the game you are currently playing,
                    // potentially reporting your scores as well.
                }*/ else if (cmd.equals("gameStop")) {
                    serverComm.gameStop(
                          itemList.getInteger("gcid"),
                          itemList.getString("category1"),
                          itemList.getString("value1"),
                          itemList.getString("category2"),
                          itemList.getString("value2"),
                          itemList.getString("category3"),
                          itemList.getString("value3"),
                          itemList.getString("category4"),
                          itemList.getString("value4"),
                          itemList.getString("presence")
                          );
                    
                    // Start a game in a particular game room/lobby.
                }/* else if (cmd.equals("gameStart")) {
                    waitingForGame = true;
                    returnValues = comm.gameStart(
                          itemList.getString("lobbyID"),
                          itemList.getInteger("lobbyMaxUsers"),
                          itemList.getString("gameRoomID"),
                          itemList.getInteger("gameRoomMaxUsers"),
                          itemList.getInteger("mode"),
                          itemList.getInteger("leader")
                          );
                   
                    // Send a game command to your opponent(s)
                }*/ else if (cmd.equals("sendGamePacket")) {
                    serverComm.sendGamePacket(itemList.getByteArray("data"));
                
                    // Log out
                } else if (cmd.equals("unifiedLogout")) {
                    serverComm.unifiedLogout();
                    
                    // Unrecognized command
                } else {
                    throw new RuntimeException("Unbekanntes Snap Kommando: " + cmd);
                }
                
                // Listener das Ergebnis des Kommandos mitteilen
                boolean handled = false;
                listener = (CommunityCommandListener)itemList.getItem("listener");
                if (listener != null) {
                    handled = listener.commandCompleted(cmd, lastError, lastErrorSeverity, returnValues);
                }
                
                // Asynchronen Fehler-Listener fr dieses Kommando registrieren
                asyncListener = (CommunityCommandListener)itemList.getItem("asyncListener");
                if (asyncListener != null) asyncCommand = cmd;
                
                //Ein Fehler der nicht zum Abbruch fhrt ist aufgetreten
                if (!handled && lastErrorSeverity == SnapEventListener.SEVERITY_NON_FATAL) {
                    showError( lastError);
                }
                //waitingForGame = false;
                
            }
        } catch (Exception e) {
            e.printStackTrace();
            showError("Es ist ein nicht behabbarer Fehler aufgetreten. Die Anwendung muss beendet werden.");
            e.printStackTrace();
            try { Thread.sleep( 5000); } catch (Exception ignore) {}
        }
        
        System.out.println("SNAP-Kommando-Schleife beendet!");
    }
    private void showError(String msg){
        game.uiManager.showLastErrorDialog(msg);
    }
    
    // SnapEventListener interface -----------------------------------------------------------------------------
    
   /**
     * SnapEventListener callback.  Verarbeitet alle ankommenden asynkronen
     * Ereignisse, die vom SNAP Mobile servers gesendet werden
     * @param list Vector of ItemLists, each element is a separate
     *             SNAP message, event, or callback.
     */
    public void processEvents(Vector list) {
        ItemList il;
        String from, msg, gameRoom;
        byte[] data;
        String event = "";

        //System.out.println("Community.processEvents()");

        try {

            //Verarbeiten aller eingegangenen Nachrichten
            for (int i = 0; i < list.size(); i++) {
 
                il = (ItemList) list.elementAt(i);

                switch (il.getInteger("id")) {

                    // Buddy messages
                    case ItemList.IMPS_IM_MESSAGE:
               /*         //System.out.println("got imps message");

                        from = clipName(il.getString("fromName"));
                        msg = il.getString("message");

                        // Challenge requests are just specially formatted
                        // buddy messages.
                        if (msg.startsWith(CHALLENGE_PREFIX)) {
                            gameRoom = msg.substring(CHALLENGE_PREFIX.length());
                            //System.out.println("got challenge message, game room is " + gameRoom);

                            showDialog("Challenge",
                                    "Player '" + from + "' is challenging you to a game. Accept?",
                                    gameRoom, null, Dialog.YES_NO
                            );
                        } else {
                            View view = findView(CHAT);

                            if (view instanceof Chat) {
                                Chat chat = (Chat) view;
                                chat.addBuddyMessage(from, msg);
                            } else {
                                buddyList.addChat(from, msg);
                            }
                        } */
                        break;

                        // Presence Update fr "FRIENDS"
                    case ItemList.IMPS_IM_PRESENCE:
                      /*  handlePresenceUpdate(il);*/
                        break;

                        // Anfrage von "Buddy", ob er in die Liste der "FRIENDS"
                        // aufgenommen werden kann
                    case ItemList.IMPS_BUDDY_REQ:
                     /*   from = clipName(il.getString("fromUserName"));
                        msg = il.getString("message");
                        showDialog("Friend Request", "From " + from + ". Accept?", from, null, Dialog.YES_NO);*/
                        break;

                        // Fhre Anfrage wird von BUDDY besttigt oder abgelehnt
                    case ItemList.IMPS_BUDDY_ACC_REJ:
                      /*  int accepted = il.getInteger("accepted");
                        from = clipName(il.getString("fromUserName"));

                        if (accepted != 0) buddyList.add(new Buddy(from, Buddy.ONLINE_AVAILABLE));
                        showDialog(
                                "Note", "'" + from + "' " +
                                (accepted != 0 ? "accepted" : "rejected") +
                                " buddy request.", null, null, Dialog.ALERT);*/
                        break;

                        // Spieler wurde aus einer FRIENDS-Liste entfernt
                        // friends list.
                    case ItemList.IMPS_BUDDY_REVOKED:
                       /* from = clipName(il.getString("fromUserName"));
                        showDialog("Note", "'" + from + "' is no longer your buddy.", null, null, Dialog.ALERT);
                        buddyList.remove(from);*/
                        break;

                        // Neues game packet angekommen
                    case ItemList.GAME_PACKET:
                        from = clipName(il.getString("from"));
                        data = il.getByteArray("gameData");

                        if (this.netSynchronizer!=null) this.netSynchronizer.gameDataReceived(from, data);
                        break;
                }
            }
        } catch (Exception e) {
            System.out.println("Exception in CommunityManager.handleEvents()!");
            e.printStackTrace();
        }
    }
     /**
     * Namen des Gegeners aus dem erhaltenen String
     * heraussuchen
     * @param name
     */
    private String clipName(String name) {
        int index = name.indexOf('@');

        if (index < 0) return name;
        return name.substring(0, index);
    }


    /**
     * SnapEventListener callback. Alle SNAP Mobile Server Fehler
     * werden hier empfangen, sowohl synchrone als auch asynchrone Fehler
     * Die Methode speichert die Server-Fehler, damit sie spter in der
     * Kommandoschleife verarbeitet werden knnen. Einige Dialoge verarbeiten
     * die Fehler selbsts (via commandCompleted()), nicht behandelte
     * Fehler werden schlielich in einen allgemeinen Dialog angezeigt
     *
     * @param error    Error number to process
     * @param msg      Error Message
     * @param severity Severity of the Error Message
     */
    public void processServerError(int error, String msg, int severity) {
//        System.out.println("Community.processServerError(), error = " + error + ", msg = " + msg + ", severity = " + severity);

        synchronized (this) {
            lastError = msg + " " + error;
            lastErrorSeverity = severity;
        }

 
        // Schwerer Fehler -- SessionId ist auf Server nicht mehr gltig
        // Neuer Login notwendig.
        if (severity == SEVERITY_FATAL) {
             synchronized (this) {
                cmdList.removeAllElements();
            }
            msg = msg + "Schwerer Fehler! Bitte Appplikation erneut starten!";
            showError(msg);
        }
        
        // Fehler erfordert einen Logout.
        else if (severity == SEVERITY_REQUIRES_LOGOUT) {
            synchronized (this) {
                cmdList.removeAllElements();
            }
            if (error == 901) {
                if (msg == null) msg = "Sie knnten zweimal eingeloggt sein! (901)";
            }
        }
        
        // bertragungsfehler
        else if (severity == SEVERITY_TRANSPORT) {
            showError(msg + " - Ein Netzwerkfehler ist aufgetreten! " +
                    "Bitte prfen Sie die Strke des bertragungssignals. ");
        }
        
        // Warnung von ServerComm, dass das Netzwerk etwas langsam ist
        else if (severity == SEVERITY_TRANSPORT_WARNING) {
  
        }
        
        // Der Fehler bezieht sich auf das letzte Kommando, das abgesetzt wurde
        // Informationen wird an entsprechenden Listener zur Fehlerbehandlung
        // weitergegeben.
        else if (severity == SEVERITY_NON_FATAL) {
            if (asyncListener != null) {
                asyncListener.commandCompleted(asyncCommand, lastError, severity, new ItemList());
            }
        }
    }
    
  // SnapEventListener interface -----------------------------------------------------------------------------
    
  public boolean commandCompleted(String cmd, String errorMessage, int errorSeverity, ItemList results) {
  

            if (cmd.equals("extendedLogin")) {
                resultLogin(errorMessage,results);
                return true;
            }
            // If we started a game successfully, switch to game view.
            else if (cmd.equals("gameStart") || cmd.equals("randomStart")) {
                resultQuickMatch(errorMessage,results);
                return true;
            }
            // Logout.
            else if (cmd.equals("unifiedLogout")) {
          /*      setLoginInfo(null, null, null, null);
                switchToView(Community.WELCOME);*/
                return true;
            }
            if (cmd.equals("randomStart")) {
                if (errorMessage.equals("CmdTimeout 762")) {
                    showError("Keine Mitspieler gefunden!");
                    return true;
                }
               // if (!waitingForGame) return true;
            } else if (cmd.equals("gameStart")) {
              /*  if (errorMessage.equals("CmdTimeout 762") && waitingForGame) {
                    showError("Sorry, the player didn't respond to your " +
                            "challenge in time.  Please try again!"
                    );
                    return true;
                }
                if (!waitingForGame) return true;*/
            } else if (cmd.equals("requestBuddy")) {
                /*if (errorMessage.equals("Not Found 914")) {
                    showError("Sorry, couldn't find a user by that name. " +
                            "Please try another name!"
                    );
                    return true;
                }*/
            }
        
        return false;
    }
  
  
    synchronized void destroyApp() {
        cmdList.removeAllElements();
        if (isLoggedIn()) {
            ItemList itemList = new ItemList();
            itemList.setItem("cmd", "unifiedLogout");
            itemList.setItem("listener", this);
            addCmd(itemList);
        }
        if (serverComm != null) {
            serverComm.removeSnapEventListener(this);
            serverComm.stop();
        }
        cmdLoop = null;
    }

    public Integer getGCID() {
        return this.gcid;
    }

  
}
