/*
 * Spiele programmieren mit Java ME
 * von (c) Gerhard Vlkl (Autor) 
 * Dpunkt Verlag; Auflage: 1 (April 2008) ISBN 389864488X 
 */

package mgame;

import com.nokia.sm.net.ItemList;
import java.util.Random;
import mgame.ui.GameDialog;

/**
 * NelsonFlow
 * @author Gerhard Vlkl
 */
public class NetSynchronizer {
    private CommunityManager communityManager; 
    private GameDialog gameDialog;
    
    private static final long CONTACT_TIMEOUT = 25000;
    private static final long PING_INTERVAL = 8000;
    private static final int MINIMAL_CHANGE_POSITION=3;
    
    private int oldX=0;
    private int oldY=0;
    private byte[] updateBuffer = new byte[9];
    
    private static final int STATE_NONE=0;
    private static final int STATE_START_WAITING_FOR_NUMBER=11;
    private static final int STATE_START_WAIT_FOR_LEAD_PING=12;
    private static final int STATE_START_WAIT_FOR_FOLLOW_ACK=13;
    private static final int STATE_START_WAIT_FOR_FOLLOW_PING=14;
    private static final int STATE_START_WAIT_FOR_LEVEL_START=15;
    private static final int STATE_GAME_RUNNING=2;
    private static final int STATE_COMPLETED_WIN=31;
    private static final int STATE_COMPLETED_LOST=32;
    
    private int state = STATE_NONE;
    
    protected static final byte NET_NUMBER = (byte) 0;
    protected static final byte NET_FOLLOW_ACK = (byte) 1;
    protected static final byte NET_LEAD_PING = (byte) 2;
    protected static final byte NET_FOLLOW_PING = (byte) 3;
    protected static final byte NET_LEVEL_START = (byte) 4;
    protected static final byte NET_PLAYER_UPDATE = (byte) 5;
    protected static final byte NET_COMPLETION_WIN = (byte) 6;
    protected static final byte NET_COMPLETION_LOST = (byte) 8;
    protected static final byte NET_CONCEDE = (byte) 7;
    protected static final byte NET_HEARTBEAT = (byte) 9;
    protected static final byte NET_EXIT_GAME = (byte) 10;

    private long lastPacketSent=0;
    private long lastContact=0;
   
    private int startNumber;
    private static Random rnd = new Random();
    private boolean isLeader;

    public NetSynchronizer(CommunityManager communityManager,
                           GameDialog gameDialog) {
        this.communityManager = communityManager;
        this.gameDialog = gameDialog;
    }

    public void starting() {
        state=STATE_START_WAITING_FOR_NUMBER;
        startNumber = getRandom(0, 100);
        System.out.println("NSynch: send NET_NUMBER");
        sendGamePacket(new byte[]{NET_NUMBER, (byte) startNumber});
    }
    
    private int getRandom(int low, int high) {
        return low + Math.abs(rnd.nextInt() % (high - low));
    }
    
     /**
      * Versenden der Multiuserinformation
     */
    private void sendGamePacket(byte[] buffer) {

        ItemList itemList = new ItemList();
        itemList.setItem("cmd", "sendGamePacket");
        itemList.setItem("data", buffer);

        if (buffer[0] == NET_PLAYER_UPDATE || buffer[0] == NET_HEARTBEAT) {
            itemList.setItem(communityManager.NON_CRITICAL, "true");
        } else {
            communityManager.cancelNonCriticalCmds();
        }

        communityManager.addCmd(itemList);
        lastPacketSent = System.currentTimeMillis();
    }
    
    /**
     * Verarbeitet die erhaltenen Multiuserinformationen
     */
    public void gameDataReceived(String from, byte[] data) {
        int id, arg, pos, rdelta;

        lastContact = System.currentTimeMillis();

        id = data[0];

        // START
        if (state == STATE_START_WAITING_FOR_NUMBER && id == NET_NUMBER) {
          //  if (startNumber == data[1]) throw new Exception("Same StartNumber!");
            System.out.println("NSynch: rcs NET_NUMER my:"+startNumber+" re:"+data[1]);
            
            isLeader = startNumber > data[1];
            if (!isLeader) {
                sendGamePacket(new byte[]{NET_FOLLOW_ACK});
                System.out.println("NSynch: send NET_FOLLOW_ACK");
                
                state = STATE_START_WAIT_FOR_LEAD_PING;
            } else {
                state = STATE_START_WAIT_FOR_FOLLOW_ACK;
            }
            gameDialog.setLeader(isLeader);
            return;
        }

        if (state == STATE_START_WAIT_FOR_FOLLOW_ACK && id == NET_FOLLOW_ACK) {
            sendGamePacket(new byte[]{NET_LEAD_PING});
            System.out.println("NSynch: send NET_LEAD_PING");
            state= STATE_START_WAIT_FOR_FOLLOW_PING;
           // then = System.currentTimeMillis();
            return;
        }

        if (state == STATE_START_WAIT_FOR_LEAD_PING && id == NET_LEAD_PING) {
            sendGamePacket(new byte[]{NET_FOLLOW_PING});
            System.out.println("NSynch: send NET_FOLLOW_PING");
            state= STATE_START_WAIT_FOR_LEVEL_START;
            return;
        }

        if (state == STATE_START_WAIT_FOR_FOLLOW_PING && id == NET_FOLLOW_PING) {
           // delay = (int) (System.currentTimeMillis() - then) / 2;
           
            sendGamePacket(new byte[]{NET_LEVEL_START});
            System.out.println("NSynch: send NET_FOLLOW_PING");
           /* try {
                Thread.sleep(delay);
            }
            catch (InterruptedException e) {
            }*/

            gameDialog.setState(gameDialog.STATE_RUN);
            state=STATE_GAME_RUNNING;
            //levelStart = System.currentTimeMillis();
            return;
        }

        if (state == STATE_START_WAIT_FOR_LEVEL_START && id == NET_LEVEL_START) {
            //delay = 0;
            gameDialog.setState(gameDialog.STATE_RUN);
            state=STATE_GAME_RUNNING;
            //levelStart = System.currentTimeMillis();
            return;
        } 
        
    
        //Spiel luft
        if (id == NET_HEARTBEAT) {
            System.out.println("heartbeat received.");
            return;
        }
        
        if (state == STATE_GAME_RUNNING && id == NET_PLAYER_UPDATE) {
            System.out.println("NSynch: resc NET_PLAYER_UPDATE "+data[1]+" "+data[2]);
            gameDialog.setOpponentPosition(unpackInteger(data,1),unpackInteger(data,5));
            return;
        }
        
         
        //Abbruch
        if (id == NET_EXIT_GAME) {
            gameDialog.setState(gameDialog.STATE_INTERRUPTED);
            gameOver();
            return;
        }
        
        
        //Regulres Ende
        if (state == STATE_GAME_RUNNING && id == NET_COMPLETION_WIN) {
            sendGamePacket(new byte[]{NET_CONCEDE});
            gameDialog.setState(gameDialog.STATE_LOST);
            gameStopLost();
            return;
        }
        
        if (state == STATE_GAME_RUNNING && id == NET_COMPLETION_LOST) {
            sendGamePacket(new byte[]{NET_CONCEDE});
            gameDialog.setState(gameDialog.STATE_WIN);
            gameStopWin();
            return;
        }
        
        if (state == STATE_COMPLETED_LOST && id == NET_CONCEDE) {
            gameDialog.setState(gameDialog.STATE_LOST);
            gameStopLost();
            return;
        }
        
        if (state == STATE_COMPLETED_WIN && id == NET_CONCEDE) {
            gameDialog.setState(gameDialog.STATE_WIN);
            gameStopWin();
        }

        System.out.println("received stray packet, id = " + id);
    }

    private void gameStopLost(){
        ItemList itemList = new ItemList();
        itemList.setItem("cmd", "gameStop");
        itemList.setItem("gcid", communityManager.getGCID());
        itemList.setItem("category1", "LOSSES");
        itemList.setItem("value1", "1");

        communityManager.addCmd(itemList);
        return;
    }
    
    private void gameStopWin(){
        ItemList itemList;

        itemList = new ItemList();
        itemList.setItem("cmd", "gameStop");
        itemList.setItem("gcid", communityManager.getGCID());
        itemList.setItem("category2", "WINS");
        itemList.setItem("value2", "1");

        communityManager.addCmd(itemList);
    }
    
    public void update(int x, int y, int gameState) {
        
        //Verbindung aufrecht erhalten
        if (lastContact > 0) {
            if ((System.currentTimeMillis() - lastContact) >= CONTACT_TIMEOUT) {
                gameDialog.setState(GameDialog.STATE_INTERRUPTED);
                gameOver();
            }
        }
        if (lastPacketSent > 0 &&
                (System.currentTimeMillis() - lastPacketSent) > PING_INTERVAL) {
            sendGamePacket(new byte[]{NET_HEARTBEAT});
        }
        
        //nderung am Spiel-Modell austauschen
        if(Math.abs(x-oldX)>=MINIMAL_CHANGE_POSITION||
           Math.abs(y-oldY)>=MINIMAL_CHANGE_POSITION){
            updateBuffer[0]=NET_PLAYER_UPDATE;
            packInteger(updateBuffer,1,x);
            packInteger(updateBuffer,5,y);
            sendGamePacket(updateBuffer);
            System.out.println("NSynch: send NET_PLAYER_UPDATE "+x+" "+y);
            oldX=x;
            oldY=y;
        }
        
        switch(gameState){
            case GameDialog.STATE_LOST:
                sendGamePacket(new byte[]{NET_COMPLETION_LOST});
                state=STATE_COMPLETED_LOST;
                break;
            case GameDialog.STATE_WIN:
                sendGamePacket(new byte[]{NET_COMPLETION_WIN});
                state=STATE_COMPLETED_WIN;
                break;
         }
    }
        
  private void packInteger(byte[] buf, int offset, int value) {
        for (int i = 0, shift = 24; i < 4; i++, shift -= 8) {
            buf[offset + i] = (byte) ((value >> shift) & 0xff);
        }
    }

    private int unpackInteger(byte[] buf, int offset) {
        int value = 0;

        for (int i = 0; i < 4; i++) {
            value = (value << 8) | (buf[offset + i] & 0xff);
        }

        return value;
    }
    
    private void gameOver() {
        sendGamePacket(new byte[]{NET_EXIT_GAME});

        ItemList itemList = new ItemList();
        itemList.setItem("cmd", "gameStop");
        itemList.setItem("gcid", communityManager.getGCID());
        communityManager.addCmd(itemList);
    }
}
