/*
 * Spiele programmieren mit Java ME
 * von (c) Gerhard Vlkl (Autor) 
 * Dpunkt Verlag; Auflage: 1 (April 2008) ISBN 389864488X 
 */

package mgame;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import javax.microedition.rms.InvalidRecordIDException;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import javax.microedition.rms.RecordStoreFullException;
import javax.microedition.rms.RecordStoreNotFoundException;
import javax.microedition.rms.RecordStoreNotOpenException;

/**
 * NelsonFlow
 * @author Gerhard Vlkl
 */
public class OptionManager {
    
    private RecordStore rs=null;
    private static final String RS_NAME = "OPTIONEN";
    
    /** Optionen **/
    public final static int OPTION_AUDIO_ON = 1;
    public final static int OPTION_AUDIO_VOLUME = 2;
    
    private final static String[] DEFAULT_VALUES={"ON","100"};
    
    public OptionManager() {
        
    }
    
    private RecordStore getStore() throws RecordStoreFullException,
            RecordStoreNotFoundException, RecordStoreException, IOException {
        if (rs == null) {
            rs = RecordStore.openRecordStore(RS_NAME, true);
            int num = rs.getNumRecords();
            if (rs.getNumRecords()!=DEFAULT_VALUES.length){
                for(int i=rs.getNumRecords();i<DEFAULT_VALUES.length;i++){
                    byte[] data=getBytes(DEFAULT_VALUES[i]);
                    rs.addRecord(data,0,data.length);
                }
            }
        }
        
        return rs;
    }
    
    private String setBytes(byte[] record) throws IOException {
        ByteArrayInputStream bais = new ByteArrayInputStream(record);
        DataInputStream dis = new DataInputStream(bais);
        
        String s = dis.readUTF();
        dis.close();
        return s;
    }
    
    private byte[] getBytes(String record) throws IOException{
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream das = new DataOutputStream(baos);
        das.writeUTF(record);
        das.close();
        return baos.toByteArray();
    }
    public void destroyApp() {
        try {
            rs.closeRecordStore();
            rs = null;
        } catch (Throwable e) {
            e.printStackTrace();
        }
    }
    
    public String get(int key) {
        byte[] data = null;
        String s = null;
        try {
            data = getStore().getRecord(key);
            s = setBytes(data);
        } catch (RecordStoreFullException ex) {
            ex.printStackTrace();
        } catch (InvalidRecordIDException ex) {
            ex.printStackTrace();
        } catch (RecordStoreNotFoundException ex) {
            ex.printStackTrace();
        } catch (RecordStoreNotOpenException ex) {
            ex.printStackTrace();
        } catch (RecordStoreException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        return s;
    }
    
    public void set(int key, String data) {
        
        try {
            byte[] b = getBytes(data);
            getStore().setRecord(key,b, 0, b.length);
        } catch (RecordStoreFullException ex) {
            ex.printStackTrace();
        } catch (InvalidRecordIDException ex) {
            ex.printStackTrace();
        } catch (RecordStoreNotFoundException ex) {
            ex.printStackTrace();
        } catch (RecordStoreNotOpenException ex) {
            ex.printStackTrace();
        } catch (RecordStoreException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
    }
}
