/*
 * Spiele programmieren mit Java ME
 * von (c) Gerhard Vlkl (Autor) 
 * Dpunkt Verlag; Auflage: 1 (April 2008) ISBN 389864488X 
 */

package mgame;

import java.io.IOException;
import javax.microedition.rms.InvalidRecordIDException;
import javax.microedition.rms.RecordComparator;
import javax.microedition.rms.RecordEnumeration;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import javax.microedition.rms.RecordStoreNotOpenException;

/**
 * NelsonFlow
 * @author Gerhard Vlkl
 */
public class HighScoreManager implements RecordComparator{
    
    private Game game;
    private static final String RECORDSTORE_NAME = "HighScore";
    private static final int MAX_ELEMENTS=8;

    public HighScoreManager(Game game){
        this.game = game;
    }
    
    public void checkHighScore(long newScore){
        try {
            if(isInHighScore(newScore)){
                game.uiManager.showInputHighScoreDialog(newScore);
            }
        } catch (RecordStoreException ex) {
            ex.printStackTrace();
        }
    }

    private boolean isInHighScore(long newScore) throws RecordStoreException {
        RecordStore store = RecordStore.openRecordStore(RECORDSTORE_NAME,true);
        boolean result = true;
        RecordEnumeration en = store.enumerateRecords(null,this,true);
        HighScoreRecord record = new HighScoreRecord();
        while(en.hasNextElement()){
            result = false;
            try {
                record.setBytes(en.nextRecord());
            } catch (RecordStoreNotOpenException ex) {
                ex.printStackTrace();
            } catch (InvalidRecordIDException ex) {
                ex.printStackTrace();
            } catch (IOException ex) {
                ex.printStackTrace();
            } catch (RecordStoreException ex) {
                ex.printStackTrace();
            }
            if (newScore>record.score) {
                result = true;
                break;
            }
        }
        store.closeRecordStore(); 
        return result;
    }

    public int compare(byte[] rec1, byte[] rec2) {
        int result = EQUIVALENT;
        try {
            HighScoreRecord high1 = new HighScoreRecord();
        
            high1.setBytes(rec1);
       
            HighScoreRecord high2 = new HighScoreRecord();
            high2.setBytes(rec2);
            if(high1.score > high2.score){
                result=PRECEDES;
            }
            else if (high1.score < high2.score){
                result=FOLLOWS;
            }
         } catch (IOException ex) {
            ex.printStackTrace();
        }    
        return result;
    }

    public void addScore(long score, String name) {
        RecordStore store;
        try {
            store = RecordStore.openRecordStore(RECORDSTORE_NAME, true);
        
        HighScoreRecord hsr = new HighScoreRecord();
        hsr.score = score;
        hsr.name = name;
            byte[] record;
            try {
                record = hsr.getBytes();
           
        store.addRecord(record,0,record.length);
        store.closeRecordStore(); 
         } catch (IOException ex) {
                ex.printStackTrace();
            }
        } catch (RecordStoreException ex) {
            ex.printStackTrace();
        }
    }
   
	
    public String[] getHightScores(){
        RecordStore store;
        String[] ergebnis=null;
        
        try {
            store = RecordStore.openRecordStore(RECORDSTORE_NAME, true);
       
            if (store.getNumRecords()==0) return ergebnis;
                
        ergebnis = new String[store.getNumRecords()];
        
        RecordEnumeration en = store.enumerateRecords(null,this,true);
        HighScoreRecord record = new HighScoreRecord();
        
        int i = 0;
        
        while(en.hasNextElement()){
            record.setBytes(en.nextRecord());
	    ergebnis[i] = record.toString();
            i++;
	}
         } catch (Exception ex) {
            ex.printStackTrace();
        }
	return ergebnis;
    }
}
