/*
 * Spiele programmieren mit Java ME
 * von (c) Gerhard Vlkl (Autor) 
 * Dpunkt Verlag; Auflage: 1 (April 2008) ISBN 389864488X 
 */
import java.io.IOException;
import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.game.GameCanvas;
import javax.microedition.m3g.Appearance;
import javax.microedition.m3g.Background;
import javax.microedition.m3g.Camera;
import javax.microedition.m3g.Graphics3D;
import javax.microedition.m3g.Image2D;
import javax.microedition.m3g.IndexBuffer;
import javax.microedition.m3g.Light;
import javax.microedition.m3g.Material;
import javax.microedition.m3g.Mesh;
import javax.microedition.m3g.PolygonMode;
import javax.microedition.m3g.Texture2D;
import javax.microedition.m3g.Transform;
import javax.microedition.m3g.TriangleStripArray;
import javax.microedition.m3g.VertexArray;
import javax.microedition.m3g.VertexBuffer;
import javax.microedition.m3g.World;
import mgame.Game;
import mgame.ui.GameDialog;

/**
 * Murmel - Version 1
 * @author Gerhard Vlkl
 */
public class MurmelCanvas extends GameDialog implements Runnable {
    
    /**** Game Model - Start ****/
    private int level = 1;
    private int power = 100;
    private int powerPerTick = 0;
    
    private World world=null;
    
    /**** Game Model - Ende  ****/
    volatile Thread gameLoop;
    private static final int MILLIS_PER_TICK = 50;
    
    private boolean highScoreChecked=false;
    
    private static final int COLOR_ALL = 0;
    private static final int COLOR_EMISSIVE = 1;
    private static final int COLOR_AMBIENT = 2;
    private static final int COLOR_SPECULAR = 3;
    private static final int COLOR_DIFFUSE = 4;
    private static final int MAX_COLOR_MODE=4;
    private int colorMode=0;
    int ambient = 0, diffuse = 0, specular = 0, emissive = 0;
    float shininess = 0;
    Material material=null;
    
    /**
     * Creates a new instance of MurmelCanvas
     */
    public MurmelCanvas(Game game) throws IOException{
        super(game);
        
        this.level = game.getLevel();
        init();
    }
    
    private void init() throws IOException {
        createWorld();
        //world = getRoot(this);
    }
    
    protected void showNotify(){
        gameLoop = new Thread(this);
        gameLoop.start();
    }
    
    protected void hideNotify(){
        gameLoop = null;
    }
    
    public void run() {
        Graphics g = getGraphics();
        Graphics3D g3d = Graphics3D.getInstance();
        g3d.setViewport(0,0,getWidth(),getHeight());
        
        try {
            
            Thread t = Thread.currentThread();
            
            while(t==gameLoop){
                long startTime = System.currentTimeMillis();
                
                update(getKeyStates());
                render(g,g3d);
                flushGraphics();
                long timeTaken = System.currentTimeMillis() - startTime;
                
                if (timeTaken < MILLIS_PER_TICK) { // etwas warten, damit andere auch dran kommen
                    try{ Thread.sleep(MILLIS_PER_TICK - timeTaken);} catch (InterruptedException ex){};
                }
            }
        } catch(Exception e){
            Alert info = new Alert("Exception",e.toString(),null,null);
            info.setTimeout(Alert.FOREVER);
            Display.getDisplay(game.midlet).setCurrent(info);
        }
    }
       
    private void update(int keyStates) {
        
    if(level==3){
        if((keyStates & GameCanvas.UP_PRESSED)!=0){
            colorMode = (colorMode==MAX_COLOR_MODE)? 0:  colorMode+1;
            setMaterial(colorMode);
	}
        else if ((keyStates & GameCanvas.DOWN_PRESSED)!=0){
             colorMode = (colorMode==0) ? MAX_COLOR_MODE: colorMode-1;
             setMaterial(colorMode);
        }
    }
    }
    
    private void render(Graphics g, Graphics3D g3d) {
        g3d.bindTarget(g);
        g3d.render(world);
        g3d.releaseTarget();
    }
    
    private void createWorld() {
        
        world = new World();
        
        Background background = new Background();
        background.setColor(0x000000FF);
        world.setBackground(background);
        
        Mesh plane=null;
        switch(level){
            case 1: plane=createPlane();break;
            case 2: plane=createPlane2();break;
            case 3: plane=createPlane3();break;
        }
        world.addChild(plane);
        
        //Camera
        Camera camera = new Camera();
        camera.setPerspective(60.0f,  //Field of View
                (float)getWidth()/(float)getHeight(),
                0.1f, //Near Clipping Plane
                100.0f); //Far Clipping Plane
        
        Transform cameraTransform = new Transform();
        cameraTransform.setIdentity();
        cameraTransform.postTranslate(10.0f,6.0f,12.0f); 
        cameraTransform.postRotate(40,0,1,0); //Rotation um Y-Achse
        cameraTransform.postRotate(-24,1,0,0); //Rotation um X-Achse
        
        camera.setTransform(cameraTransform);
        
        world.addChild(camera);
        world.setActiveCamera(camera);
        
        //Light
        if(level==3){
            Light lightAmbient = new Light();
            lightAmbient.setMode(Light.AMBIENT);
            Light light = new Light();
            light.setMode(Light.OMNI);
            light.setColor(0x00FFFFFF);
            light.setIntensity(0.5f);
            light.setTranslation(0,4,0);
            world.addChild(lightAmbient);
            world.addChild(light);
        }
        
    }
    
    private Mesh createPlane() {
        
        short[] positionArray = {
            4, 1,  4,  -4, 1, 4,   4,-1, 4,  -4,-1, 4,  // vorne
            -4, 1, -4,   4, 1,-4,  -4,-1,-4,  -4,-1,-4,  // hinten
            -4, 1,  4,  -4, 1,-4,  -4,-1,-4,  -4,-1,-4,  // links
            4, 1, -4,   4, 1, 4,   4,-1,-4,   4,-1, 4,  // rechts
            4, 1, -4,  -4, 1,-4,   4, 1, 4,  -4, 1, 4,  // oben
            4,-1,  4,  -4,-1, 4,   4,-1,-4,  -4,-1,-4   // unten
        };
        
        VertexArray postionVertexArray = new VertexArray(positionArray.length/3,3,2);
        postionVertexArray.set(0,positionArray.length/3,positionArray);
        
        byte c = (byte) 255;
        byte[] colorArray = {
            c, 0, 0,  c, 0, 0,  c, 0, 0,  c, 0, 0,  // vorne
            0, 0, c,  0, 0, c,  0, 0, c,  0, 0, c,  // hinten
            0, c, 0,  0, c, 0,  0, c, 0,  0, c, 0,  // links
            c, c, 0,  c, c, 0,  c, c, 0,  c, c, 0,  // rechts
            c, c, c,  c, c, c,  c, c, c,  c, c, c,  // oben
            c, 0, c,  c, 0, c,  c, 0, c,  c, 0, c   // unten
        };
        
        VertexArray vertexColors = new VertexArray(colorArray.length/3, 3, 1);
        vertexColors.set(0, colorArray.length/3, colorArray);
        
        //VertexBuffer
        VertexBuffer vertexBuffer = new VertexBuffer();
        float bias[] = { 0.00f, 0.00f, 0.00f};
        vertexBuffer.setPositions(postionVertexArray,1.00f,bias);
        vertexBuffer.setColors(vertexColors);
        
        //length of TriangleStrips
        int[] stripLength ={4,4,4,4,4,4};
        
        //IndexBuffer
       /* int[] indices = {
            1,2,0,3,5,6,4,7,9,10,8,11,13,14,12,15,17,18,16,19,21,22,20,23};*/
        int[] indices = {
            0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23};
        IndexBuffer indexBuffer=new TriangleStripArray(indices,stripLength);
        
        PolygonMode polygonMode = new PolygonMode();
        polygonMode.setCulling(PolygonMode.CULL_BACK);
        polygonMode.setShading(PolygonMode.SHADE_SMOOTH);
        polygonMode.setWinding(168);
        polygonMode.setTwoSidedLightingEnable(false);
        polygonMode.setLocalCameraLightingEnable(false);
        polygonMode.setPerspectiveCorrectionEnable(true);
        
        //Appearance
        Appearance appearance = new Appearance();
        appearance.setPolygonMode(polygonMode);
        
        //Mesh
        Mesh mesh = new Mesh(vertexBuffer,indexBuffer,appearance);
        
        return mesh;
    }
    
    private Mesh createPlane2() {
        
        short[] positionArray = {
            4, 1,  4,  -4, 1, 4,   4,-1, 4,  -4,-1, 4,  // vorne
            -4, 1, -4,   4, 1,-4,  -4,-1,-4,  -4,-1,-4,  // hinten
            -4, 1,  4,  -4, 1,-4,  -4,-1,-4,  -4,-1,-4,  // links
            4, 1, -4,   4, 1, 4,   4,-1,-4,   4,-1, 4,  // rechts
            4, 1, -4,  -4, 1,-4,   4, 1, 4,  -4, 1, 4,  // oben
            4,-1,  4,  -4,-1, 4,   4,-1,-4,  -4,-1,-4   // unten
        };
        
        VertexArray postionVertexArray = new VertexArray(positionArray.length/3,3,2);
        postionVertexArray.set(0,positionArray.length/3,positionArray);
        
        //Texturkoordinat
        short[] textureArray = {
            1,0, 0,0, 1,1, 0,1,
            1,0, 0,0, 1,1, 0,1,
            1,0, 0,0, 1,1, 0,1,
            1,0, 0,0, 1,1, 0,1,
            1,0, 0,0, 1,1, 0,1,
            1,0, 0,0, 1,1, 0,1
        };
        
        VertexArray textureVertexArray = new VertexArray(textureArray.length/2,
                2,2);
        textureVertexArray.set(0,textureArray.length/2,textureArray);
        
        //VertexBuffer
        VertexBuffer vertexBuffer = new VertexBuffer();
        float bias[] = { 0.00f, 0.00f, 0.00f};
        vertexBuffer.setPositions(postionVertexArray,1.00f,bias);
        vertexBuffer.setTexCoords(0,textureVertexArray,1.0f,null);
        
        //length of TriangleStrips
        int[] stripLength ={4,4,4,4,4,4};
        
        //IndexBuffer
        int[] indices = {
            1,2,0,3,5,6,4,7,9,10,8,11,13,14,12,15,17,18,16,19,21,22,20,23};
        
        IndexBuffer indexBuffer=new TriangleStripArray(0,stripLength);
        
        PolygonMode polygonMode = new PolygonMode();
        polygonMode.setCulling(PolygonMode.CULL_BACK);
        polygonMode.setShading(PolygonMode.SHADE_SMOOTH);
        polygonMode.setWinding(168);
        polygonMode.setTwoSidedLightingEnable(false);
        polygonMode.setLocalCameraLightingEnable(false);
        polygonMode.setPerspectiveCorrectionEnable(true);
        
        //Texture
        Image image=null;
        try {
            image = Image.createImage("/texture.png");
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
        Image2D image2D = new Image2D(Image2D.RGB,image);
        
        Texture2D texture = new Texture2D(image2D);
        texture.setFiltering(Texture2D.FILTER_NEAREST,
                Texture2D.FILTER_NEAREST);
        texture.setWrapping(Texture2D.WRAP_CLAMP,Texture2D.WRAP_CLAMP);
        texture.setBlending(Texture2D.FUNC_MODULATE);
        
        //Appearance
        Appearance appearance = new Appearance();
        appearance.setPolygonMode(polygonMode);
        appearance.setTexture(0,texture);
        
        //Mesh
        Mesh mesh = new Mesh(vertexBuffer,indexBuffer,appearance);
        
        return mesh;
    }
    
    private void setMaterial(int colorMode) {
        int ambientConst  = 0x333333;     // grau
        int diffuseConst  = 0xFFFFFF;     // weiss
        int specularConst = 0xFFFFFF;     // weiss
        int emissivConst  = 0x333333;     // grau
        float shininessConst = 15;
        
        ambient = 0; diffuse = 0; specular = 0; emissive = 0;
        shininess = 0;
        
        switch (colorMode) {
            case COLOR_AMBIENT:
                ambient = ambientConst;
                break;
                
            case COLOR_DIFFUSE:
                diffuse = diffuseConst;
                break;
                
            case COLOR_EMISSIVE:
                emissive = emissivConst;
                break;
                
            case COLOR_SPECULAR:
                specular = diffuseConst;
                shininess = shininessConst;
                break;
                
            case COLOR_ALL:
                ambient = ambientConst;
                diffuse = diffuseConst;
                emissive = emissivConst;
                specular = specularConst;
                shininess = shininessConst;
                break;
        }
        if(this.material!=null){
            material.setColor(Material.AMBIENT,  ambient);
            material.setColor(Material.DIFFUSE,  diffuse);
            material.setColor(Material.SPECULAR, specular);
            material.setColor(Material.EMISSIVE, emissive);
            material.setShininess(shininess);
        }
    }
    
    private Mesh createPlane3() {
        short[] positionArray = {
            4, 1,  4,  -4, 1, 4,   4,-1, 4,  -4,-1, 4,  // vorne
            -4, 1, -4,   4, 1,-4,  -4,-1,-4,  -4,-1,-4,  // hinten
            -4, 1,  4,  -4, 1,-4,  -4,-1,-4,  -4,-1,-4,  // links
            4, 1, -4,   4, 1, 4,   4,-1,-4,   4,-1, 4,  // rechts
            4, 1, -4,  -4, 1,-4,   4, 1, 4,  -4, 1, 4,  // oben
            4,-1,  4,  -4,-1, 4,   4,-1,-4,  -4,-1,-4   // unten
        };
        
        VertexArray postionVertexArray = new VertexArray(positionArray.length/3,3,2);
        postionVertexArray.set(0,positionArray.length/3,positionArray);
        
        //normal
        byte[] normalArray = {
            0,    0, -127,      0,    0, -127,     0,    0, -127,     0,    0, -127,
            0,    0,  127,      0,    0,  127,     0,    0,  127,     0,    0,  127,
            127,    0,    0,    127,    0,    0,   127,    0,    0,   127,    0,    0,
            0, -127,    0,      0, -127,    0,     0, -127,    0,     0, -127,    0,
            -127,    0,    0,   -127,    0,    0,  -127,    0,    0,  -127,    0,    0,
            0,  127,    0,      0,  127,    0,     0,  127,    0,     0,  127,    0
        };
        
        VertexArray normalVertexArray = new VertexArray(normalArray.length/3,3,1);
        normalVertexArray.set(0,normalArray.length/3,normalArray);
        
        //VertexBuffer
        VertexBuffer vertexBuffer = new VertexBuffer();
        float bias[] = { 0.00f, 0.00f, 0.00f};
        vertexBuffer.setPositions(postionVertexArray,1.00f,bias);
        vertexBuffer.setNormals(normalVertexArray);
        
        //length of TriangleStrips
        int[] stripLength ={4,4,4,4,4,4};
        
        //IndexBuffer
        int[] indices = {
            1,2,0,3,5,6,4,7,9,10,8,11,13,14,12,15,17,18,16,19,21,22,20,23};
        
        IndexBuffer indexBuffer=new TriangleStripArray(0,stripLength);
        
        PolygonMode polygonMode = new PolygonMode();
        polygonMode.setCulling(PolygonMode.CULL_BACK);
        polygonMode.setShading(PolygonMode.SHADE_SMOOTH);
        polygonMode.setWinding(168);
        polygonMode.setTwoSidedLightingEnable(false);
        polygonMode.setLocalCameraLightingEnable(false);
        polygonMode.setPerspectiveCorrectionEnable(true);
        
        //Material
        Material material = new Material();
        material.setVertexColorTrackingEnable(false);
        setMaterial(COLOR_ALL);
        material.setColor(Material.AMBIENT,  ambient);
        material.setColor(Material.DIFFUSE,  diffuse);
        material.setColor(Material.SPECULAR, specular);
        material.setColor(Material.EMISSIVE, emissive);
        material.setShininess(shininess);
        this.material = material;
        
        //Appearance
        Appearance appearance = new Appearance();
        appearance.setPolygonMode(polygonMode);
        appearance.setMaterial(material);
        
        //Mesh
        Mesh mesh = new Mesh(vertexBuffer,indexBuffer,appearance);
        
        return mesh;
    }
    
}
