/*
 * Spiele programmieren mit Java ME
 * von (c) Gerhard Vlkl (Autor) 
 * Dpunkt Verlag; Auflage: 1 (April 2008) ISBN 389864488X 
 */
/**
 * Cube3D
 * @author Gerhard Vlkl
 */
import javax.microedition.lcdui.game.GameCanvas;
import java.util.*;
import javax.microedition.lcdui.*;
import javax.microedition.m3g.*;

class Canvas3D extends GameCanvas implements Runnable{
	private final Cube3D _Midlet;
	
	private Thread _Thread = null;
	
	private volatile boolean _isPlay = true;
	
	Graphics3D _G3D;
  Camera _Camera;
  Light _Light;
  private VertexBuffer _VertexBuffer;
  IndexBuffer _IndexBuffer;
  Image _Image;
  Image2D _Image2D;
  Texture2D _Texture;
  Appearance _Appearance;
  Material _Material;
  PolygonMode _PolygonMode;
  Background _Background;
  Transform _Transform = new Transform();
  float _Angle = 0.0f; 
  private static final short[] vert = {
           10, 10, 10,  -10, 10, 10,   10,-10, 10,  -10,-10, 10,  // vorne
          -10, 10,-10,   10, 10,-10,  -10,-10,-10,  -10,-10,-10,  // hinten
          -10, 10, 10,  -10, 10,-10,  -10,-10,-10,  -10,-10,-10,  // links
           10, 10,-10,   10, 10, 10,   10,-10,-10,   10,-10, 10,  // rechts
           10, 10,-10,  -10, 10,-10,   10, 10, 10,  -10, 10, 10,  // oben
           10,-10, 10,  -10,-10, 10,   10,-10,-10,  -10,-10,-10 }; //unten 
           
           
	Canvas3D(Cube3D aMidlet){
		super(true);
		_Midlet = aMidlet;
	}
	
	public void start(){  //startet oder restartet Spiel
		_isPlay = true;
		
		_Thread = new Thread(this); //Game-Loop starten (run)
		_Thread.start();	
	}
	
	public void stop(){
		_isPlay = false;
	}
	
	public void run(){
		Graphics g = getGraphics();
		
		//Game-Loop
		while(_isPlay){
	    	if (isShown()){
	    		
	    		draw3D(g);
	    		
	    		flushGraphics();
	    	}		
	    	//etwas warten, damit andere auch dran kommen
	    	try { _Thread.sleep(20); }
            catch (InterruptedException ie) {}
		}
	}
	
	
	void init(){
		 //Verbinung zur 3D-Grafik
     _G3D = Graphics3D.getInstance();

     //Erzeuge Kamera
     _Camera = new Camera();
     _Camera.setPerspective(60.0f,  //Field of View
                            (float)this.getWidth()/(float)this.getHeight(), //Seitenverhltnis
                            1.0f, //Near Clipping Plane
                            1000.0f); //Far Clipping Plane

     //Erzeuge Licht
     _Light = new Light();
     _Light.setColor(0xffffff); //Farbe: wei
     _Light.setIntensity(1.25f); //sehr hell

     //Erzeuge Cube
     //Vertizes - Je Seite des Wrfels
     VertexArray vertexArray = new VertexArray(vert.length/3,3,2);
     vertexArray.set(0,vert.length/3,vert); //in VertexArray speichern

     //Normalvektoren je Vertex
     byte[] normal = {
            0,   0, 127,     0,   0, 127,     0,   0, 127,     0,   0, 127,
            0,   0,-127,     0,   0,-127,     0,   0,-127,     0,   0,-127,
         -127,   0,   0,  -127,   0,   0,  -127,   0,   0,  -127,   0,   0,
          127,   0,   0,   127,   0,   0,   127,   0,   0,   127,   0,   0,
            0, 127,   0,     0, 127,   0,     0, 127,   0,     0, 127,   0,
            0,-127,   0,     0,-127,   0,     0,-127,   0,     0,-127,   0};

     VertexArray normalArray = new VertexArray(normal.length/3,
                                               3,1);
     normalArray.set(0,normal.length/3,normal);  //in VertexArray speichern

     //Texturkoordinat
     short[] texture = {
         1,0, 0,0, 1,1, 0,1,
         1,0, 0,0, 1,1, 0,1,
         1,0, 0,0, 1,1, 0,1,
         1,0, 0,0, 1,1, 0,1,
         1,0, 0,0, 1,1, 0,1,
         1,0, 0,0, 1,1, 0,1};

     VertexArray textureArray = new VertexArray(texture.length/2,
                                                2,2);
     textureArray.set(0,texture.length/2,texture);

     //Lnges jedes TriangleStrip
     int[] stripLength ={4,4,4,4,4,4};

     //VertexBuffer erzeugen
     _VertexBuffer = new VertexBuffer();
     _VertexBuffer.setPositions(vertexArray,1.0f,null);
     _VertexBuffer.setNormals(normalArray);
     _VertexBuffer.setTexCoords(0,textureArray,1.0f,null);

     //IndexBuffer erzeugen
     _IndexBuffer=new TriangleStripArray(0,stripLength);

     //Bild fr Textur laden
     try{
       _Image = Image.createImage("/texture.png");
     }
     catch(Exception e)
     {
       System.out.println("IO Exception:texture.png");
       _Midlet.quitApp();
     }

     _Image2D = new Image2D(Image2D.RGB,_Image);

     //Textur erzeugen
     Texture2D lTexture = new Texture2D(_Image2D);
     lTexture.setFiltering(Texture2D.FILTER_NEAREST,
                           Texture2D.FILTER_NEAREST);
     lTexture.setWrapping(Texture2D.WRAP_CLAMP,Texture2D.WRAP_CLAMP);
     lTexture.setBlending(Texture2D.FUNC_MODULATE);

     //Appearance erzeugen
     _Appearance = new Appearance();
     _Appearance.setTexture(0,lTexture);
     _Material = new Material();
     _Appearance.setMaterial(_Material);
     _Material.setColor(Material.DIFFUSE,0xFFFFFFFF); //weiss
     _Material.setColor(Material.SPECULAR,0xFFFFFFFF); //weiss
     _Material.setShininess(100.0f);
     _PolygonMode = new PolygonMode();
     _PolygonMode.setPerspectiveCorrectionEnable(true);
     _Appearance.setPolygonMode(_PolygonMode);

     //Background erzeugen
     _Background = new Background();
   //  _Background.setColor(0xf00000);
     // _Background.setColor(0xf54588);

	start();
	}
	
	void draw3D (Graphics g){

    if (_G3D==null) return;
   try{
    _G3D.bindTarget(g,true,
                    Graphics3D.DITHER |
                    Graphics3D.TRUE_COLOR);
    _G3D.clear(_Background);

    //Kamera in Position bringen
    Transform lTransform = new Transform();
    lTransform.postTranslate(0.0f,0.0f,30.0f);
    _G3D.setCamera(_Camera,lTransform);

    //Licht
    _G3D.resetLights();
    _G3D.addLight(_Light,lTransform);

    //Rotieren
    _Angle += 1.0f;
    _Transform.setIdentity();
    _Transform.postRotate(_Angle,  //Winkel
                          1.0f,1.0f,1.0f);  //Rotationsachse

    //Rendern
    _G3D.render(_VertexBuffer,_IndexBuffer,_Appearance,_Transform);

    //Freigeben
    _G3D.releaseTarget();
   }
   catch (Exception e){
     e.printStackTrace();
    _Midlet.quitApp();
   }
  }
}