/*
 * Spiele programmieren mit Java ME
 * von (c) Gerhard Vlkl (Autor) 
 * Dpunkt Verlag; Auflage: 1 (April 2008) ISBN 389864488X 
 */
import javax.microedition.lcdui.game.GameCanvas;
import javax.microedition.m3g.Mesh;
import javax.microedition.m3g.Node;
import javax.microedition.m3g.Transform;
import mgame.Math3D;

/**
 * Murmel - Version 2
 * @author Gerhard Vlkl
 */
public class BoardActor {
    
    public Node node;
    public float rotateX = 0.0f;
    public float rotateZ = 0.0f;
    private static final float MAX_ROTATE=20.0f;
    private static final float MIN_ROTATE=-20.0f;
    private static final float ROTATE_STEP=1.0f;
    private int[][] map;
    public static final int MAP_EMPTY    = 0;
    public static final int MAP_GROUND   = 1;
    public static final int MAP_WIN      = 2;
    public static final int MAP_BUILDING_1 = 3;
    public static final int MAP_BUILDING_2 = 4;
    private int mapHeight=0;
    private int mapWidth=0;
    private float minX=0.0f;
    private float minZ=0.0f;
    
    public BoardActor(Node node, Node mesh, 
                      int[][] map, int width, int height) {
        this.node = node;
        this.map = map;
        mapHeight=height;
        mapWidth=width;
        float[] aabb = new float[8];
        Math3D.calculateAABB((Mesh) mesh, aabb);
        float[] pos = new float[3];
        Math3D.calulateRelativPosition(node,mesh,pos);
        minX=aabb[0]-pos[0];
        minZ=aabb[2]-pos[2];
    }
    
    void update(int keyStates) {
        boolean changed = false;
        if((keyStates & GameCanvas.LEFT_PRESSED)!=0){
            rotateX=Math.min(rotateX+ROTATE_STEP,MAX_ROTATE);
            changed = true;
	}
        else if((keyStates & GameCanvas.RIGHT_PRESSED)!=0){
            rotateX=Math.max(rotateX-ROTATE_STEP,MIN_ROTATE);
            changed = true;
    	}
        else if((keyStates & GameCanvas.UP_PRESSED)!=0){
            rotateZ=Math.min(rotateZ+ROTATE_STEP,MAX_ROTATE);
            changed = true;
	}
        else if ((keyStates & GameCanvas.DOWN_PRESSED)!=0){
            rotateZ=Math.max(rotateZ-ROTATE_STEP,MIN_ROTATE);
            changed = true;
        }
        if(changed){
            Transform t = new Transform();
            t.postRotate(rotateX,1,0,0);
            t.postRotate(rotateZ,0,0,1);
            node.setTransform(t);
        }
    }
    
    public int szi;
    public int sxi;
    
    public int getMapType(float posX, float posZ) {
       float z = posZ-minZ;
       if (z<0) return MAP_EMPTY;
       int zi = (int)Math.floor(z);
       szi=zi;
       if (zi>mapHeight) return MAP_EMPTY;
       float x = posX-minX;
       if (x<0) return MAP_EMPTY;
       int xi = (int) Math.floor(x+0.5);
       sxi=xi;
       if (xi>mapWidth) return MAP_EMPTY;
       return map[zi][xi];
    }
}
