/*
 * Spiele programmieren mit Java ME
 * von (c) Gerhard Vlkl (Autor) 
 * Dpunkt Verlag; Auflage: 1 (April 2008) ISBN 389864488X 
 */
import java.io.IOException;
import javax.microedition.lcdui.Graphics;
import javax.microedition.m3g.Graphics3D;
import javax.microedition.m3g.Loader;
import javax.microedition.m3g.Node;
import javax.microedition.m3g.Object3D;
import javax.microedition.m3g.World;
import mgame.Game;
import mgame.Math3D;
import mgame.ui.GameDialog;

/**
 * Murmel - Version 2
 * @author Gerhard Vlkl
 */
public class MurmelCanvas extends GameDialog implements Runnable {
    
    /**** Game Model - Start ****/
    private int level = 1;
    private static final int SPHERE_ID=1;
    private static final int BOARD_ID =2;
    private static final int INNER_SPHERE_ID=3;
    private static final int BUILDING_1_ID=4;
    private static final int BUILDING_2_ID=5;
    private static final int INNER_BOARD_ID=6;
    private BoardActor board;
    private SphereActor sphere;
    private BuildingActor building_1;
    private BuildingActor building_2;
    private World world=null;
    private CameraManager cameraManager=null;
    private int power=0;
    private int powerPart=1000;
    
    /**** Game Model - Ende  ****/
    
    volatile Thread gameLoop;
    private static final int MILLIS_PER_TICK = 50;
    private static final float MILLIS_PER_TICK_IN_SEC = MILLIS_PER_TICK/1000.0f;
    private static final float MILLIS_PER_TICK_IN_SEC_SQUARE = MILLIS_PER_TICK_IN_SEC * MILLIS_PER_TICK_IN_SEC;
    
    private int debugDelay=0;
    private boolean highScoreChecked=false;
    
    public MurmelCanvas(Game game) throws IOException{
        super(game);
        
        this.level = game.getLevel();
        init();
    }
    
    Node innerSphere=null;
    
    float[] posSphere = new float[3];
    
    private void init() throws IOException {
        power=120;
        Object3D[] parts=null;
        parts = Loader.load("/Murmel_"+String.valueOf(level)+".m3g");
        world=(World) parts[0];
        
        int[][] map_level_1 = {{4,4,1,1,1,1,1,1,1,1,1,1,1,1,1},
        {2,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
        {3,3,1,1,1,1,1,1,1,1,1,1,1,1,1}};
        board = new BoardActor((Node)world.find(BOARD_ID),
                               (Node)world.find(INNER_BOARD_ID),
                                map_level_1,14,2);
        sphere = new SphereActor((Node)world.find(SPHERE_ID),board.node);
        
        cameraManager = new CameraManager(world.getActiveCamera(),this);
        
        innerSphere= (Node) world.find(INNER_SPHERE_ID);
        
        building_1 = new BuildingActor((Node)world.find(BUILDING_1_ID));
        float[] pos = new float[4];
        Math3D.calulateRelativPosition(board.node, building_1.node,pos);
        building_2 = new BuildingActor((Node)world.find(BUILDING_2_ID));
        
        this.gameState = this.STATE_RUN;
    }
    
    protected void showNotify(){
        gameLoop = new Thread(this);
        gameLoop.start();
    }
    
    protected void hideNotify(){
        gameLoop = null;
    }
    
    public void run() {
        Graphics g = getGraphics();
        Graphics3D g3d = Graphics3D.getInstance();
        g3d.setViewport(0,0,getWidth(),getHeight());
        
        try {
            
            Thread t = Thread.currentThread();
            
            while(t==gameLoop){
                long startTime = System.currentTimeMillis();
                switch(gameState){
                    case STATE_RUN:
                        update(getKeyStates());
                        render(g,g3d);
                        break;
                    case STATE_LOST:
                        render(g,g3d);
                        g.setColor(255,255,255);
                        g.drawString("VERLOREN!",getWidth()/2,getHeight()/2,Graphics.HCENTER| Graphics.TOP);
                        break;
                    case STATE_WIN:
                        render(g,g3d);
                        g.setColor(255,255,255);
                        g.drawString("GEWONNEN!",getWidth()/2,getHeight()/2,Graphics.HCENTER| Graphics.TOP);
                        break;
                }
                
                flushGraphics();
                long timeTaken = System.currentTimeMillis() - startTime;
                
                if (timeTaken < MILLIS_PER_TICK) { // etwas warten, damit andere auch dran kommen
                    try{ Thread.sleep(MILLIS_PER_TICK - timeTaken);} catch (InterruptedException ex){};
                } else debugDelay +=1;
            }
        } catch(Exception e){
            game.showException(e);
        }
    }
    
    private void testHighScore(){
        if(!highScoreChecked){
            game.highScoreManager.checkHighScore(power);
            highScoreChecked=true;
        }
    }
    
    private void update(int keyStates) {
        //Power
        powerPart -= MILLIS_PER_TICK;
        if (powerPart<=0){
            powerPart=1000;
            power-=1;
            if (power<0){
                gameState=STATE_LOST;
                return;
            }
        }
        //Eingabe des Spielers
        board.update(keyStates);
        
        //Kugel nach den Spielregeln und der Spielephysik bewegen 
        float[] acceleration = {0.0f, 0.0f, 0.0f};
        
        // Beschleunigung ermitteln
        if (sphere.isFalling){
            acceleration[1] = -9.81f;
        } else{
            acceleration[0] = (float) Math.sin(Math.toRadians(-1.0f*board.rotateZ)) * 9.81f * sphere.mass;
            acceleration[2] = (float) Math.sin(Math.toRadians(board.rotateX)) * 9.81f * sphere.mass;
        }
        
        //Geschwindigkeit ermitteln
        sphere.velocity[0] += acceleration[0] * MILLIS_PER_TICK_IN_SEC;
        sphere.velocity[1] += acceleration[1] * MILLIS_PER_TICK_IN_SEC;
        sphere.velocity[2] += acceleration[2] * MILLIS_PER_TICK_IN_SEC;
        
        if (sphere.velocity[0]==0.0f && sphere.velocity[1]==0.0f
                && sphere.velocity[2]==0.0f) return;
        
        //Neue Position berechnen   
        float[] position = {sphere.velocity[0] * MILLIS_PER_TICK_IN_SEC+ 0.5f * sphere.velocity[0]*MILLIS_PER_TICK_IN_SEC_SQUARE,
        sphere.velocity[1] * MILLIS_PER_TICK_IN_SEC+ 0.5f * sphere.velocity[1]*MILLIS_PER_TICK_IN_SEC_SQUARE,
        sphere.velocity[2] * MILLIS_PER_TICK_IN_SEC+ 0.5f * sphere.velocity[2]*MILLIS_PER_TICK_IN_SEC_SQUARE};
        sphere.addPostion(position);
        
        //Rotation um Achse
        float[] rotAxis = Math3D.vectorCrossProduct(position,new float[] {0,1,0});
        float degrees = (float) Math.toDegrees(Math3D.vectorLength(position)*-2.0f);
        innerSphere.postRotate(degrees,rotAxis[0],rotAxis[1],rotAxis[2]);
        
        
        if (!sphere.isFalling){
            //Kollisionserkennung: Broad Phase
            switch(board.getMapType(sphere.position[0],
                    sphere.position[2])){
                case BoardActor.MAP_WIN:
                    //Ziel erreicht !
                    this.gameState = STATE_WIN;
                    break;
                case BoardActor.MAP_EMPTY:
                    //Die Kugel fllt
                    sphere.isFalling = true;
                    break;
                case BoardActor.MAP_GROUND:
                    break;
                    //Kollisionserkennung: Narrow Phase
                case BoardActor.MAP_BUILDING_1:
                    if (Math3D.isCollidingSphereAABB(sphere.position,sphere.radius,
                            building_1.boundingBox)){
                        sphere.colliding();
                    }
                    break;
                case BoardActor.MAP_BUILDING_2:
                    if (Math3D.isCollidingSphereAABB(sphere.position,sphere.radius,
                            building_2.boundingBox)){
                        sphere.colliding();
                    }
                    break;
            }
        } else {
            //Kugel aus Sichtweite ? --> Spielende
            if (sphere.position[1] < -5){
                this.gameState = STATE_LOST;
            }
        }
        
        // Kamera
        cameraManager.update(board);
    }
    
    private void render(Graphics g, Graphics3D g3d) {
        g3d.bindTarget(g);
        g3d.render(world);
        g3d.releaseTarget();
        draw2D(g);
    }
    
    private void draw2D(Graphics g){
        
        //Color
        g.setColor(255,255,255);
        
        //Power
        g.drawString("Power: " + power,
                     getWidth(),
                     0,
                     Graphics.TOP | Graphics.RIGHT);
        
        //Delay-Info
     /*   g.drawString("info: delay: "+debugDelay+" x: "+board.sxi+" z:"+board.szi,
                0,
                getHeight(),
                Graphics.BOTTOM | Graphics.LEFT); */
        
    }
}
