/*
 * Spiele programmieren mit Java ME
 * von (c) Gerhard Vlkl (Autor) 
 * Dpunkt Verlag; Auflage: 1 (April 2008) ISBN 389864488X 
 */
package mgame;

import javax.microedition.m3g.Mesh;
import javax.microedition.m3g.Node;
import javax.microedition.m3g.Transform;
import javax.microedition.m3g.VertexArray;
import javax.microedition.m3g.VertexBuffer;

/**
 * Murmel - Version 2
 * @author Gerhard Vlkl
 */
public final class Math3D {
    
    private Math3D() {
    }
    
    public static float[] vectorCrossProduct(float[] a, float[] b) {
        float[] result = {a[1] * b[2] - a[2] * b[1],
        a[2] * b[0] - a[0] * b[2],
        a[0] * b[1] - a[1] * b[0]};
        return result;
    }
    
    public static float vectorLength(float[] a) {
        return (float)Math.sqrt(a[0] * a[0] + a[1] * a[1] + a[2] * a[2]);
    }
    
    public static void calculateAABB(Mesh mesh, float[]aabb){
        //Koordinaten der Vertex ermitteln
        VertexBuffer vertexBuffer = mesh.getVertexBuffer();
        float[] scaleBias = new float[4];
        
        VertexArray vertexArray = vertexBuffer.getPositions(scaleBias);
        float [] vertexCoordinates = new float[vertexBuffer.getVertexCount()*4];
        Transform t = new Transform();
        t.setIdentity();
        t.postTranslate(scaleBias[1], scaleBias[2], scaleBias[3]);
        t.postScale(scaleBias[0],scaleBias[0],scaleBias[0]);
        t.transform(vertexArray, vertexCoordinates, true);
        
        Transform tNode = new Transform();
        mesh.getCompositeTransform(tNode);
        tNode.transform(vertexCoordinates);
        
        //Maximum und Minimum jeder Dimension ermitteln
        for(int j=0;j<3;j++){
            aabb[j]   = vertexCoordinates[j]; //Min
            aabb[j+4] = vertexCoordinates[j]; //Max
            for(int i=0;i<vertexBuffer.getVertexCount();i++){
                float element = vertexCoordinates[i*4+j];
                if (element < aabb[j]){ aabb[j]=element;} else if (element >aabb[j+4]){aabb[j+4]=element;}
            }
        }
        //Fr homogene Koordinaten
        aabb[3]=1.0f;
         aabb[7]=1.0f;
        
        //Transformation mit einkalkulieren
       /* Transform tNode = new Transform();
        mesh.getCompositeTransform(tNode);
        tNode.transform(aabb); */
    }
   
    public static void calulateRelativPosition(Node myWorld, Node node, float[] position){
        Transform trans = new Transform();
        node.getTransformTo(myWorld,trans);
        float pos[] = new float[16];
        trans.get(pos);
        position[0] = pos[3];  // X
        position[1] = pos[7];  // Y
        position[2] = pos[11]; // Z 
    }
    
    // Algorithmus nach Arvo
    public static boolean isCollidingSphereAABB(float[] spherePosition, float radius, float[] aabb){
        float distance = 0.0f;
        float diff = 0.0f;
   
        for(int i=0 ; i<3 ; i++ ) {
            if( spherePosition[i] < aabb[i] ) {
                diff = spherePosition[i] - aabb[i];
                distance += diff*diff;
            } else if( spherePosition[i] > aabb[i+4] ) {
                diff = spherePosition[i] - aabb[i+4];
                distance += diff*diff;
            }
        }
        return distance <= radius*radius;
    }
}
